@@ -42,9 +42,12 @@
 
 #define MOO_EDIT_ACCEL_FOCUS_DOC "<Alt>C"
 #define MOO_EDIT_ACCEL_MOVE_TO_SPLIT_NOTEBOOK "<Alt>L"
+#define MOO_EDIT_ACCEL_FOCUS_SPLIT_NOTEBOOK "<Ctrl>L"
 #define MOO_EDIT_ACCEL_SWITCH_TO_TAB "<Alt>"
 #define MOO_EDIT_ACCEL_PREV_TAB "<Alt>Left"
 #define MOO_EDIT_ACCEL_NEXT_TAB "<Alt>Right"
+#define MOO_EDIT_ACCEL_PREV_TAB_IN_VIEW "<Alt>O"
+#define MOO_EDIT_ACCEL_NEXT_TAB_IN_VIEW "<Alt>P"
 
 #else /* GDK_WINDOWING_QUARTZ */
 
@@ -57,9 +60,12 @@
 
 #define MOO_EDIT_ACCEL_FOCUS_DOC "<Meta>J"
 #define MOO_EDIT_ACCEL_MOVE_TO_SPLIT_NOTEBOOK ""
+#define MOO_EDIT_ACCEL_FOCUS_SPLIT_NOTEBOOK ""
 #define MOO_EDIT_ACCEL_SWITCH_TO_TAB "<Meta>"
 #define MOO_EDIT_ACCEL_PREV_TAB "<Meta>Left"
 #define MOO_EDIT_ACCEL_NEXT_TAB "<Meta>Right"
+#define MOO_EDIT_ACCEL_PREV_TAB_IN_VIEW "<Meta>O"
+#define MOO_EDIT_ACCEL_NEXT_TAB_IN_VIEW "<Meta>P"
 
 #endif /* GDK_WINDOWING_QUARTZ */
 
