@@ -1,7 +1,7 @@
 /*
  *   mooedit-private.h
  *
- *   Copyright (C) 2004-2010 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -17,26 +17,30 @@
 #define MOO_EDIT_PRIVATE_H
 
 #include "mooedit/mooedit-impl.h"
+#include "mooedit/mooeditprogress.h"
 
 G_BEGIN_DECLS
 
-#define PROGRESS_TIMEOUT    100
-#define PROGRESS_WIDTH      300
-#define PROGRESS_HEIGHT     100
-
 #define MOO_EDIT_IS_UNTITLED(edit) (!(edit)->priv->file)
 
 struct MooEditPrivate {
     MooEditor *editor;
 
+    GtkTextBuffer *buffer;
+    MooEditViewArray *views;
+    MooEditView *active_view;
+    gboolean dead_active_view;
+
     gulong modified_changed_handler_id;
     guint apply_config_idle;
+    gboolean in_recheck_config;
 
     /***********************************************************************/
     /* Document
      */
     GFile *file;
     char *filename;
+    char *norm_name;
     char *display_filename;
     char *display_basename;
 
@@ -45,20 +49,11 @@ struct MooEditPrivate {
     MooEditStatus status;
 
     guint file_monitor_id;
-    gulong focus_in_handler_id;
     gboolean modified_on_disk;
     gboolean deleted_from_disk;
 
-    /***********************************************************************/
-    /* Progress dialog and stuff
-     */
     MooEditState state;
-    guint progress_timeout;
-    GtkWidget *progress;
-    GtkWidget *progressbar;
-    char *progress_text;
-    GDestroyNotify cancel_op;
-    gpointer cancel_data;
+    MooEditProgress *progress;
 
     /***********************************************************************/
     /* Bookmarks
@@ -73,6 +68,7 @@ struct MooEditPrivate {
     MooActionCollection *actions;
 };
 
+void    _moo_edit_remove_untitled   (MooEdit    *doc);
 
 G_END_DECLS
 
