@@ -34,7 +34,6 @@
  * @MOO_EDIT_STATUS_CLEAN: doesn't prompt on close, even if it's modified.
  **/
 
-#define MOOEDIT_COMPILATION
 #include "mooedit/mooeditaction-factory.h"
 #include "mooedit/mooedit-private.h"
 #include "mooedit/mooeditview-impl.h"
@@ -836,6 +835,8 @@ moo_edit_get_tab (MooEdit *doc)
 
 /**
  * moo_edit_get_view:
+ *
+ * Get the active view of this document.
  **/
 MooEditView *
 moo_edit_get_view (MooEdit *doc)
@@ -851,6 +852,8 @@ moo_edit_get_view (MooEdit *doc)
 
 /**
  * moo_edit_get_views:
+ *
+ * Get the list of views which belong to this document.
  **/
 MooEditViewArray *
 moo_edit_get_views (MooEdit *doc)
@@ -861,6 +864,8 @@ moo_edit_get_views (MooEdit *doc)
 
 /**
  * moo_edit_get_n_views:
+ *
+ * Get the number of views which belong to this document.
  **/
 int
 moo_edit_get_n_views (MooEdit *doc)
