@@ -1,16 +1,18 @@
 /*
  *   moolangmgr.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define MOOEDIT_COMPILATION
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -21,7 +23,7 @@
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooprefs.h"
 #include "marshals.h"
-#include "mooutils/xdgmime/xdgmime.h"
+#include "mooutils/moo-mime.h"
 #include <string.h>
 
 #define LANGUAGE_DIR            "language-specs"
@@ -31,7 +33,7 @@
 #define ELEMENT_MIME_TYPES      "mime-types"
 #define ELEMENT_CONFIG          "config"
 #define PROP_LANG_ID            "id"
-#define SCHEME_DEFAULT          "kate"
+#define SCHEME_DEFAULT          "medit"
 
 
 typedef struct {
@@ -56,14 +58,13 @@ static MooLang *get_lang_for_mime_type  (MooLangMgr *mgr,
                                          const char *mime_type);
 
 
-G_DEFINE_TYPE (MooLangMgr, moo_lang_mgr, GTK_TYPE_SOURCE_LANGUAGE_MANAGER)
+G_DEFINE_TYPE (MooLangMgr, moo_lang_mgr, G_TYPE_OBJECT)
 
 
 static void
 moo_lang_mgr_init (MooLangMgr *mgr)
 {
     char **dirs;
-    guint n_dirs;
 
     mgr->schemes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 
@@ -72,8 +73,9 @@ moo_lang_mgr_init (MooLangMgr *mgr)
     mgr->config = g_hash_table_new_full (g_str_hash, g_str_equal,
                                          g_free, g_free);
 
-    dirs = moo_get_data_subdirs (LANGUAGE_DIR, MOO_DATA_SHARE, &n_dirs);
-    g_object_set (mgr, "search-path", dirs, NULL);
+    mgr->lang_mgr = gtk_source_language_manager_new ();
+    dirs = moo_get_data_subdirs (LANGUAGE_DIR);
+    g_object_set (mgr->lang_mgr, "search-path", dirs, NULL);
     mgr->style_mgr = gtk_source_style_scheme_manager_new ();
     gtk_source_style_scheme_manager_set_search_path (mgr->style_mgr, dirs);
 
@@ -90,6 +92,7 @@ moo_lang_mgr_dispose (GObject *object)
 
     if (mgr->langs)
     {
+        g_object_unref (mgr->lang_mgr);
         g_object_unref (mgr->style_mgr);
         g_hash_table_destroy (mgr->langs);
         g_hash_table_destroy (mgr->config);
@@ -122,7 +125,7 @@ moo_lang_mgr_class_init (MooLangMgrClass *klass)
 MooLangMgr *
 moo_lang_mgr_new (void)
 {
-    return g_object_new (MOO_TYPE_LANG_MGR, NULL);
+    return MOO_LANG_MGR (g_object_new (MOO_TYPE_LANG_MGR, (const char*) NULL));
 }
 
 MooLangMgr *
@@ -215,7 +218,7 @@ moo_lang_mgr_get_lang (MooLangMgr *mgr,
     lang = _moo_lang_mgr_find_lang (mgr, name);
 
     if (!lang)
-        g_warning ("could not find language '%s'", name);
+        g_message ("could not find language '%s'", name);
 
     return lang;
 }
@@ -254,7 +257,7 @@ get_lang_by_extension (MooLangMgr *mgr,
                        const char *filename)
 {
     MooLang *lang = NULL;
-    char *basename, *utf8_basename;
+    char *basename;
     GSList *langs, *l;
     gboolean found = FALSE;
 
@@ -262,11 +265,8 @@ get_lang_by_extension (MooLangMgr *mgr,
 
     read_langs (mgr);
 
-    /* TODO: is this right? */
     basename = g_path_get_basename (filename);
     g_return_val_if_fail (basename != NULL, NULL);
-    utf8_basename = g_filename_display_name (basename);
-    g_return_val_if_fail (utf8_basename != NULL, NULL);
 
     langs = moo_lang_mgr_get_available_langs (mgr);
 
@@ -279,7 +279,7 @@ get_lang_by_extension (MooLangMgr *mgr,
 
         for (g = globs; !found && g != NULL; g = g->next)
         {
-            if (_moo_glob_match_simple ((char*) g->data, utf8_basename))
+            if (_moo_glob_match_simple ((char*) g->data, basename))
             {
                 found = TRUE;
                 break;
@@ -294,7 +294,6 @@ get_lang_by_extension (MooLangMgr *mgr,
 
     g_slist_foreach (langs, (GFunc) g_object_unref, NULL);
     g_slist_free (langs);
-    g_free (utf8_basename);
     g_free (basename);
     return lang;
 }
@@ -305,7 +304,7 @@ lang_mgr_get_lang_for_bak_filename (MooLangMgr *mgr,
                                     const char *filename)
 {
     MooLang *lang = NULL;
-    char *utf8_name, *utf8_base = NULL;
+    char *base = NULL;
     int len;
     guint i;
 
@@ -313,32 +312,23 @@ lang_mgr_get_lang_for_bak_filename (MooLangMgr *mgr,
 
     read_langs (mgr);
 
-    utf8_name = g_filename_display_name (filename);
-    len = strlen (utf8_name);
+    len = strlen (filename);
 
     for (i = 0; i < G_N_ELEMENTS (bak_globs); ++i)
     {
         int ext_len = strlen (bak_globs[i]) - 1;
 
-        if (len > ext_len && _moo_glob_match_simple (bak_globs[i], utf8_name))
+        if (len > ext_len && _moo_glob_match_simple (bak_globs[i], filename))
         {
-            utf8_base = g_strndup (utf8_name, len - ext_len);
+            base = g_strndup (filename, len - ext_len);
             break;
         }
     }
 
-    if (utf8_base)
-    {
-        char *base = g_filename_from_utf8 (utf8_base, -1, NULL, NULL, NULL);
+    if (base)
+        lang = get_lang_for_filename (mgr, base);
 
-        if (base)
-            lang = get_lang_for_filename (mgr, base);
-
-        g_free (base);
-        g_free (utf8_base);
-    }
-
-    g_free (utf8_name);
+    g_free (base);
     return lang;
 }
 
@@ -348,7 +338,7 @@ filename_blacklisted (MooLangMgr *mgr,
                       const char *filename)
 {
     /* XXX bak files */
-    char *basename, *utf8_basename;
+    char *basename;
     gboolean result = FALSE;
     LangInfo *info;
     GSList *l;
@@ -357,17 +347,14 @@ filename_blacklisted (MooLangMgr *mgr,
 
     basename = g_path_get_basename (filename);
     g_return_val_if_fail (basename != NULL, FALSE);
-    utf8_basename = g_filename_display_name (basename);
-    g_return_val_if_fail (utf8_basename != NULL, FALSE);
 
     info = get_lang_info (mgr, MOO_LANG_NONE, FALSE);
 
     if (info)
         for (l = info->globs; !result && l != NULL; l = l->next)
-            if (_moo_glob_match_simple ((char*) l->data, utf8_basename))
+            if (_moo_glob_match_simple ((char*) l->data, basename))
                 result = TRUE;
 
-    g_free (utf8_basename);
     g_free (basename);
     return result;
 }
@@ -383,40 +370,43 @@ file_blacklisted (MooLangMgr *mgr,
 
 MooLang *
 moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
-                                const char *filename)
+                                GFile      *file)
 {
     MooLang *lang = NULL;
     const char *mime_type;
+    char *filename;
 
     g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    g_return_val_if_fail (filename != NULL, NULL);
+    g_return_val_if_fail (G_IS_FILE (file), NULL);
 
     read_langs (mgr);
 
+    filename = g_file_get_parse_name (file);
+
     if (file_blacklisted (mgr, filename))
-        return NULL;
+        goto out;
 
     lang = get_lang_by_extension (mgr, filename);
 
     if (lang)
-        return lang;
+        goto out;
 
     lang = lang_mgr_get_lang_for_bak_filename (mgr, filename);
 
     if (lang)
-        return lang;
+        goto out;
 
-    /* XXX: xdgmime wants utf8-encoded filename here. is it a problem? */
-    /* It's a big problem! */
-    mime_type = xdg_mime_get_mime_type_for_file (filename, NULL);
+    mime_type = moo_get_mime_type_for_file (filename, NULL);
 
-    if (mime_type != XDG_MIME_TYPE_UNKNOWN)
+    if (mime_type != MOO_MIME_TYPE_UNKNOWN)
         lang = get_lang_for_mime_type (mgr, mime_type);
 
     if (lang)
-        return lang;
+        goto out;
 
-    return NULL;
+out:
+    g_free (filename);
+    return lang;
 }
 
 
@@ -440,12 +430,9 @@ get_lang_for_filename (MooLangMgr *mgr,
     if (lang)
         return lang;
 
-    /* XXX: xdgmime wants utf8-encoded filename here. is it a problem? */
-    /* It's a big problem! */
+    mime_type = moo_get_mime_type_for_filename (filename);
 
-    mime_type = xdg_mime_get_mime_type_from_file_name (filename);
-
-    if (mime_type != XDG_MIME_TYPE_UNKNOWN)
+    if (mime_type != MOO_MIME_TYPE_UNKNOWN)
         lang = get_lang_for_mime_type (mgr, mime_type);
 
     if (lang)
@@ -464,7 +451,7 @@ static int
 check_mime_subclass (const char *base,
                      const char *mime)
 {
-    return !xdg_mime_mime_type_subclass (mime, base);
+    return !moo_mime_type_is_subclass (mime, base);
 }
 
 
@@ -525,9 +512,7 @@ read_langs (MooLangMgr *mgr)
 
     mgr->got_langs = TRUE;
 
-#ifdef MOO_USE_XML
-    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
-#endif
+    ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
 
     get_lang_info (mgr, MOO_LANG_NONE, TRUE);
 
@@ -538,7 +523,7 @@ read_langs (MooLangMgr *mgr)
         GtkSourceLanguage *lang;
 
         id = *ids;
-        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), id);
+        lang = gtk_source_language_manager_get_language (mgr->lang_mgr, id);
 
         info = get_lang_info (mgr, id, TRUE);
         info->lang = g_object_ref (lang);
@@ -611,7 +596,7 @@ load_lang_node (MooLangMgr    *mgr,
         {
             /* 'extensions' is leftover from previous versions */
             if (strcmp (node->name, "extensions") != 0)
-                g_warning ("%s: ignoring node '%s'", G_STRLOC, node->name);
+                g_warning ("ignoring node '%s'", node->name);
         }
     }
 }
@@ -627,7 +612,7 @@ set_default_config (MooLangMgr *mgr)
 static void
 load_config (MooLangMgr *mgr)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root, *node;
 
     set_default_config (mgr);
@@ -636,8 +621,7 @@ load_config (MooLangMgr *mgr)
     xml = moo_prefs_get_markup (MOO_PREFS_RC);
     g_return_if_fail (xml != NULL);
 
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml),
-                                   ELEMENT_LANG_CONFIG);
+    root = moo_markup_get_element (xml, ELEMENT_LANG_CONFIG);
 
     if (!root)
         return;
@@ -649,7 +633,7 @@ load_config (MooLangMgr *mgr)
 
         if (strcmp (node->name, ELEMENT_LANG))
         {
-            g_warning ("%s: invalid '%s' element", G_STRLOC, node->name);
+            g_warning ("invalid '%s' element", node->name);
             continue;
         }
 
@@ -687,8 +671,7 @@ _moo_lang_mgr_set_active_scheme (MooLangMgr *mgr,
         scheme = g_hash_table_lookup (mgr->schemes, SCHEME_DEFAULT);
 
     if (!scheme)
-        g_warning ("%s: could not find style scheme '%s'",
-                   G_STRLOC, name ? name : "<NULL>");
+        g_message ("could not find style scheme '%s'", name ? name : "<NULL>");
     else
         mgr->active_scheme = scheme;
 }
@@ -736,7 +719,7 @@ read_schemes (MooLangMgr *mgr)
 
         if (!scheme)
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
         }
         else
         {
@@ -760,9 +743,7 @@ moo_lang_mgr_get_sections (MooLangMgr *mgr)
 
     read_langs (mgr);
 
-#ifdef MOO_USE_XML
-    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
-#endif
+    ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
 
     while (ids && *ids)
     {
@@ -771,7 +752,7 @@ moo_lang_mgr_get_sections (MooLangMgr *mgr)
         const char *id;
 
         id = *ids;
-        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), id);
+        lang = gtk_source_language_manager_get_language (mgr->lang_mgr, id);
         section = _moo_lang_get_section (MOO_LANG (lang));
 
         if (section && !g_slist_find_custom (sections, section, (GCompareFunc) strcmp))
@@ -794,14 +775,12 @@ moo_lang_mgr_get_available_langs (MooLangMgr *mgr)
 
     read_langs (mgr);
 
-#ifdef MOO_USE_XML
-    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
-#endif
+    ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
 
     while (ids && *ids)
     {
         GtkSourceLanguage *lang;
-        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), *ids);
+        lang = gtk_source_language_manager_get_language (mgr->lang_mgr, *ids);
         if (lang)
             list = g_slist_prepend (list, g_object_ref (lang));
         ids++;
@@ -997,7 +976,7 @@ _moo_lang_mgr_set_config (MooLangMgr *mgr,
     id = _moo_lang_id_from_name (lang_id);
     old = g_hash_table_lookup (mgr->config, id);
 
-    if (!_moo_str_equal (old, norm))
+    if (!moo_str_equal (old, norm))
         mgr->modified = TRUE;
 
     g_hash_table_insert (mgr->config, id, norm);
@@ -1056,7 +1035,7 @@ save_one (const char *lang_id,
 
     struct {
         MooLangMgr *mgr;
-        MooMarkupDoc *xml;
+        MooMarkupNode *xml;
         MooMarkupNode *root;
     } *data;
 
@@ -1067,8 +1046,7 @@ save_one (const char *lang_id,
         return;
 
     if (!data->root)
-        data->root = moo_markup_create_element (MOO_MARKUP_NODE (data->xml),
-                                                ELEMENT_LANG_CONFIG);
+        data->root = moo_markup_create_element (data->xml, ELEMENT_LANG_CONFIG);
 
     lang_node = moo_markup_create_element (data->root, ELEMENT_LANG);
     moo_markup_set_prop (lang_node, PROP_LANG_ID, lang_id);
@@ -1076,14 +1054,14 @@ save_one (const char *lang_id,
     if (info->globs_modified)
     {
         char *string = list_to_string (info->globs);
-        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string ? string : "");
+        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string);
         g_free (string);
     }
 
     if (info->mime_types_modified)
     {
         char *string = list_to_string (info->mime_types);
-        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string ? string : "");
+        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string);
         g_free (string);
     }
 
@@ -1094,12 +1072,12 @@ save_one (const char *lang_id,
 void
 _moo_lang_mgr_save_config (MooLangMgr *mgr)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root;
 
     struct {
         MooLangMgr *mgr;
-        MooMarkupDoc *xml;
+        MooMarkupNode *xml;
         MooMarkupNode *root;
     } data;
 
@@ -1112,7 +1090,7 @@ _moo_lang_mgr_save_config (MooLangMgr *mgr)
     g_return_if_fail (xml != NULL);
 
     mgr->modified = FALSE;
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml), ELEMENT_LANG_CONFIG);
+    root = moo_markup_get_element (xml, ELEMENT_LANG_CONFIG);
 
     if (root)
         moo_markup_delete_node (root);
