@@ -1,29 +1,52 @@
 /*
  *   mooplugin.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooPlugin: (parent GObject) (moo.lua 0) (moo.private 1)
+ **/
+
+/**
+ * class:MooWinPlugin: (parent GObject) (moo.lua 0) (moo.private 1)
+ **/
+
+/**
+ * class:MooDocPlugin: (parent GObject) (moo.lua 0) (moo.private 1)
+ **/
+
+/**
+ * boxed:MooPluginInfo: (moo.lua 0) (moo.private 1)
+ **/
+
+/**
+ * boxed:MooPluginParams: (moo.lua 0) (moo.private 1)
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include "mooedit/mooplugin.h"
 #include "mooedit/mooplugin-loader.h"
-#include "glade/moopluginprefs-glade.h"
-#include "mooedit/plugins/mooeditplugins.h"
+#include "mooedit/moopluginprefs-gxml.h"
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/moohelp.h"
-#include "help-sections.h"
+#include "mooutils/mootype-macros.h"
+#include "moo-help-sections.h"
 #include <string.h>
 #include <gmodule.h>
 #include <gobject/gvaluecollector.h>
@@ -49,6 +72,8 @@ static PluginStore *plugin_store = NULL;
 #define MOO_PLUGIN_QUARK plugin_store->plugin_quark
 #define MOO_PLUGIN_METHS_QUARK plugin_store->meths_quark
 
+MOO_DEFINE_BOXED_TYPE_C (MooPluginInfo, moo_plugin_info)
+MOO_DEFINE_BOXED_TYPE_C (MooPluginParams, moo_plugin_params)
 
 static void     plugin_store_init       (void);
 static void     plugin_store_add        (MooPlugin      *plugin);
@@ -186,9 +211,6 @@ moo_plugin_finalize (GObject *object)
 {
     MooPlugin *plugin = MOO_PLUGIN (object);
 
-    if (plugin->langs)
-        g_hash_table_destroy (plugin->langs);
-
     g_free (plugin->id);
     moo_plugin_info_free (plugin->info);
     moo_plugin_params_free (plugin->params);
@@ -205,6 +227,14 @@ moo_plugin_class_init (MooPluginClass *klass)
 }
 
 
+/**
+ * moo_plugin_register: (moo.lua 0) (moo.private 1)
+ *
+ * @id: (type const-utf8)
+ * @type:
+ * @info:
+ * @params: (allow-none) (default NULL)
+ **/
 gboolean
 moo_plugin_register (const char            *id,
                      GType                  type,
@@ -214,7 +244,8 @@ moo_plugin_register (const char            *id,
     MooPluginClass *klass;
     MooPlugin *plugin;
     char *prefs_key;
-    GSList *l, *windows;
+    MooEditWindowArray *windows;
+    guint i;
 
     g_return_val_if_fail (id != NULL && id[0] != 0, FALSE);
     g_return_val_if_fail (g_utf8_validate (id, -1, NULL), FALSE);
@@ -226,65 +257,28 @@ moo_plugin_register (const char            *id,
 
     if (moo_plugin_registered (type))
     {
-        g_warning ("%s: plugin '%s' already registered",
-                   G_STRLOC, g_type_name (type));
+        g_warning ("plugin '%s' already registered",
+                   g_type_name (type));
         return FALSE;
     }
 
     if (!plugin_info_check (info))
     {
-        g_warning ("%s: invalid info in plugin '%s'",
-                   G_STRLOC, g_type_name (type));
+        g_warning ("invalid info in plugin '%s'",
+                   g_type_name (type));
         return FALSE;
     }
 
-    plugin = g_object_new (type, NULL);
+    plugin = MOO_PLUGIN (g_object_new (type, (const char*) NULL));
     plugin->id = g_strdup (id);
     plugin->info = moo_plugin_info_copy ((MooPluginInfo*) info);
     plugin->params = params ? moo_plugin_params_copy ((MooPluginParams*) params) :
                               moo_plugin_params_new (TRUE, TRUE);
 
-    if (info->langs)
-    {
-        char **langs, **p;
-        GHashTable *table;
-
-        /* XXX */
-        langs = g_strsplit_set (info->langs, " \t\r\n", 0);
-        table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-
-        for (p = langs; p && *p; ++p)
-        {
-            char *lang_id;
-
-            if (!**p)
-                continue;
-
-            lang_id = _moo_lang_id_from_name (*p);
-
-            if (!g_hash_table_lookup (table, lang_id))
-                g_hash_table_insert (table, lang_id, lang_id);
-            else
-                g_free (lang_id);
-        }
-
-        if (!g_hash_table_size (table))
-        {
-            g_warning ("%s: invalid langs string '%s'", G_STRLOC, info->langs);
-            g_hash_table_destroy (table);
-        }
-        else
-        {
-            plugin->langs = table;
-        }
-
-        g_strfreev (langs);
-    }
-
     if (moo_plugin_lookup (moo_plugin_id (plugin)))
     {
-        _moo_message ("%s: plugin with id %s already registered",
-                      G_STRLOC, moo_plugin_id (plugin));
+        _moo_message ("plugin with id %s already registered",
+                      moo_plugin_id (plugin));
         g_object_unref (plugin);
         return FALSE;
     }
@@ -304,10 +298,10 @@ moo_plugin_register (const char            *id,
 
     plugin_store_add (plugin);
 
-    windows = moo_editor_list_windows (plugin_store->editor);
-    for (l = windows; l != NULL; l = l->next)
-        plugin_attach_win (plugin, l->data);
-    g_slist_free (windows);
+    windows = moo_editor_get_windows (plugin_store->editor);
+    for (i = 0; i < windows->n_elms; ++i)
+        plugin_attach_win (plugin, windows->elms[i]);
+    moo_edit_window_array_free (windows);
 
     return TRUE;
 }
@@ -364,7 +358,8 @@ plugin_attach_win (MooPlugin      *plugin,
     MooWinPluginClass *wklass;
     MooWinPlugin *win_plugin;
     GType wtype;
-    GSList *l, *docs;
+    MooEditArray *docs;
+    guint i;
 
     g_return_if_fail (MOO_IS_EDIT_WINDOW (window));
     g_return_if_fail (MOO_IS_PLUGIN (plugin));
@@ -381,7 +376,7 @@ plugin_attach_win (MooPlugin      *plugin,
 
     if (wtype && g_type_is_a (wtype, MOO_TYPE_WIN_PLUGIN))
     {
-        win_plugin = g_object_new (wtype, NULL);
+        win_plugin = MOO_WIN_PLUGIN (g_object_new (wtype, (const char*) NULL));
         g_return_if_fail (win_plugin != NULL);
 
         win_plugin->plugin = plugin;
@@ -394,10 +389,10 @@ plugin_attach_win (MooPlugin      *plugin,
             g_object_unref (win_plugin);
     }
 
-    docs = moo_edit_window_list_docs (window);
-    for (l = docs; l != NULL; l = l->next)
-        plugin_attach_doc (plugin, window, l->data);
-    g_slist_free (docs);
+    docs = moo_edit_window_get_docs (window);
+    for (i = 0; i < docs->n_elms; ++i)
+        plugin_attach_doc (plugin, window, docs->elms[i]);
+    moo_edit_array_free (docs);
 }
 
 
@@ -408,7 +403,8 @@ plugin_detach_win (MooPlugin      *plugin,
     MooPluginClass *klass;
     MooWinPluginClass *wklass;
     MooWinPlugin *win_plugin;
-    GSList *l, *docs;
+    MooEditArray *docs;
+    guint i;
 
     g_return_if_fail (MOO_IS_EDIT_WINDOW (window));
     g_return_if_fail (MOO_IS_PLUGIN (plugin));
@@ -416,10 +412,10 @@ plugin_detach_win (MooPlugin      *plugin,
     if (!moo_plugin_enabled (plugin))
         return;
 
-    docs = moo_edit_window_list_docs (window);
-    for (l = docs; l != NULL; l = l->next)
-        plugin_detach_doc (plugin, window, l->data);
-    g_slist_free (docs);
+    docs = moo_edit_window_get_docs (window);
+    for (i = 0; i < docs->n_elms; ++i)
+        plugin_detach_doc (plugin, window, docs->elms[i]);
+    moo_edit_array_free (docs);
 
     win_plugin = window_get_plugin (window, plugin);
 
@@ -458,19 +454,6 @@ plugin_attach_doc (MooPlugin      *plugin,
     if (!moo_plugin_enabled (plugin))
         return;
 
-    /* XXX ! */
-    if (plugin->langs)
-    {
-        MooLang *lang;
-        const char *id;
-
-        lang = moo_text_view_get_lang (MOO_TEXT_VIEW (doc));
-        id = _moo_lang_id (lang);
-
-        if (!g_hash_table_lookup (plugin->langs, id))
-            return;
-    }
-
     plugin->docs = g_slist_prepend (plugin->docs, doc);
 
     klass = MOO_PLUGIN_GET_CLASS (plugin);
@@ -482,7 +465,7 @@ plugin_attach_doc (MooPlugin      *plugin,
 
     if (dtype && g_type_is_a (dtype, MOO_TYPE_DOC_PLUGIN))
     {
-        doc_plugin = g_object_new (dtype, NULL);
+        doc_plugin = MOO_DOC_PLUGIN (g_object_new (dtype, (const char*) NULL));
         g_return_if_fail (doc_plugin != NULL);
 
         doc_plugin->plugin = plugin;
@@ -675,6 +658,57 @@ moo_doc_plugin_lookup (const char     *plugin_id,
 }
 
 
+/**
+ * moo_win_plugin_get_window:
+ **/
+MooEditWindow *
+moo_win_plugin_get_window (MooWinPlugin *wplugin)
+{
+    g_return_val_if_fail (MOO_IS_WIN_PLUGIN (wplugin), NULL);
+    return wplugin->window;
+}
+
+/**
+ * moo_win_plugin_get_plugin:
+ **/
+MooPlugin *
+moo_win_plugin_get_plugin (MooWinPlugin *wplugin)
+{
+    g_return_val_if_fail (MOO_IS_WIN_PLUGIN (wplugin), NULL);
+    return wplugin->plugin;
+}
+
+/**
+ * moo_doc_plugin_get_window:
+ **/
+MooEditWindow *
+moo_doc_plugin_get_window (MooDocPlugin *dplugin)
+{
+    g_return_val_if_fail (MOO_IS_DOC_PLUGIN (dplugin), NULL);
+    return dplugin->window;
+}
+
+/**
+ * moo_doc_plugin_get_doc:
+ **/
+MooEdit *
+moo_doc_plugin_get_doc (MooDocPlugin *dplugin)
+{
+    g_return_val_if_fail (MOO_IS_DOC_PLUGIN (dplugin), NULL);
+    return dplugin->doc;
+}
+
+/**
+ * moo_doc_plugin_get_plugin:
+ **/
+MooPlugin *
+moo_doc_plugin_get_plugin (MooDocPlugin *dplugin)
+{
+    g_return_val_if_fail (MOO_IS_DOC_PLUGIN (dplugin), NULL);
+    return dplugin->plugin;
+}
+
+
 static gboolean
 plugin_info_check (const MooPluginInfo *info)
 {
@@ -733,7 +767,8 @@ moo_plugin_enabled (MooPlugin *plugin)
 static gboolean
 plugin_enable (MooPlugin  *plugin)
 {
-    GSList *l, *windows;
+    MooEditWindowArray *windows;
+    guint i;
 
     g_return_val_if_fail (MOO_IS_PLUGIN (plugin), FALSE);
 
@@ -750,10 +785,10 @@ plugin_enable (MooPlugin  *plugin)
         return FALSE;
     }
 
-    windows = moo_editor_list_windows (plugin_store->editor);
-    for (l = windows; l != NULL; l = l->next)
-        plugin_attach_win (plugin, l->data);
-    g_slist_free (windows);
+    windows = moo_editor_get_windows (plugin_store->editor);
+    for (i = 0; i < windows->n_elms; ++i)
+        plugin_attach_win (plugin, windows->elms[i]);
+    moo_edit_window_array_free (windows);
 
     return TRUE;
 }
@@ -762,7 +797,8 @@ plugin_enable (MooPlugin  *plugin)
 static void
 plugin_disable (MooPlugin  *plugin)
 {
-    GSList *l, *windows;
+    MooEditWindowArray *windows;
+    guint i;
 
     g_return_if_fail (MOO_IS_PLUGIN (plugin));
 
@@ -771,10 +807,10 @@ plugin_disable (MooPlugin  *plugin)
 
     g_assert (plugin->initialized);
 
-    windows = moo_editor_list_windows (plugin_store->editor);
-    for (l = windows; l != NULL; l = l->next)
-        plugin_detach_win (plugin, l->data);
-    g_slist_free (windows);
+    windows = moo_editor_get_windows (plugin_store->editor);
+    for (i = 0; i < windows->n_elms; ++i)
+        plugin_detach_win (plugin, windows->elms[i]);
+    moo_edit_window_array_free (windows);
 
     plugin_deinit (plugin);
     plugin->params->enabled = FALSE;
@@ -904,25 +940,10 @@ moo_plugin_get_dirs (void)
 }
 
 
-static void
-moo_plugin_init_builtin (void)
-{
-    _moo_file_selector_plugin_init ();
-    _moo_file_list_plugin_init ();
-    _moo_find_plugin_init ();
-#ifndef __WIN32__
-#ifdef MOO_BUILD_CTAGS
-    _moo_ctags_plugin_init ();
-#endif
-#endif
-}
-
-
 void
 moo_plugin_read_dirs (void)
 {
     char **d, **dirs;
-    guint n_dirs;
 
     plugin_store_init ();
 
@@ -931,13 +952,10 @@ moo_plugin_read_dirs (void)
 
     plugin_store->dirs_read = TRUE;
 
-    dirs = moo_get_data_subdirs (MOO_PLUGIN_DIR_BASENAME,
-                                 MOO_DATA_LIB, &n_dirs);
+    dirs = moo_get_data_and_lib_subdirs (MOO_PLUGIN_DIR_BASENAME);
     g_strfreev (plugin_store->dirs);
     plugin_store->dirs = _moo_strv_reverse (dirs);
 
-    moo_plugin_init_builtin ();
-
     for (d = plugin_store->dirs; d && *d; ++d)
         moo_plugin_read_dir (*d);
 
@@ -963,17 +981,12 @@ moo_plugin_shutdown (void)
         list = g_slist_delete_link (list, list);
     }
 
-    /* XXX */
-#if 0 && defined(MOO_USE_PYTHON)
-    _moo_python_plugin_deinit ();
-#endif
-
     plugin_store->dirs_read = FALSE;
     plugin_store->editor = NULL;
 
     if (plugin_store->list)
     {
-        g_critical ("%s: could not unregister all plugins", G_STRLOC);
+        g_critical ("could not unregister all plugins");
         g_slist_free (plugin_store->list);
         plugin_store->list = NULL;
     }
@@ -1016,35 +1029,6 @@ _moo_window_detach_plugins (MooEditWindow *window)
 }
 
 
-static void
-doc_lang_changed (MooEdit *doc)
-{
-    GSList *l;
-    MooLang *lang;
-    const char *id;
-    MooEditWindow *window = NULL;
-
-    g_return_if_fail (MOO_IS_EDIT (doc));
-
-    window = moo_edit_get_window (doc);
-    lang = moo_text_view_get_lang (MOO_TEXT_VIEW (doc));
-    id = _moo_lang_id (lang);
-
-    for (l = plugin_store->list; l != NULL; l = l->next)
-    {
-        MooPlugin *plugin = l->data;
-
-        if (moo_plugin_enabled (plugin) && plugin->langs)
-        {
-            if (g_hash_table_lookup (plugin->langs, id))
-                plugin_attach_doc (plugin, window, doc);
-            else
-                plugin_detach_doc (plugin, window, doc);
-        }
-    }
-}
-
-
 void
 _moo_doc_attach_plugins (MooEditWindow *window,
                          MooEdit       *doc)
@@ -1056,10 +1040,6 @@ _moo_doc_attach_plugins (MooEditWindow *window,
 
     plugin_store_init ();
 
-    g_signal_connect (doc, "config-notify::lang",
-                      G_CALLBACK (doc_lang_changed),
-                      plugin_store);
-
     for (l = plugin_store->list; l != NULL; l = l->next)
         plugin_attach_doc (l->data, window, doc);
 }
@@ -1076,10 +1056,6 @@ _moo_doc_detach_plugins (MooEditWindow *window,
 
     plugin_store_init ();
 
-    g_signal_handlers_disconnect_by_func (doc,
-                                          (gpointer) doc_lang_changed,
-                                          plugin_store);
-
     for (l = plugin_store->list; l != NULL; l = l->next)
         plugin_detach_doc (l->data, window, doc);
 }
@@ -1093,6 +1069,9 @@ moo_plugin_visible (MooPlugin *plugin)
 }
 
 
+/**
+ * moo_plugin_set_info:
+ */
 void
 moo_plugin_set_info (MooPlugin     *plugin,
                      MooPluginInfo *info)
@@ -1107,6 +1086,9 @@ moo_plugin_set_info (MooPlugin     *plugin,
 }
 
 
+/**
+ * moo_plugin_set_doc_plugin_type:
+ */
 void
 moo_plugin_set_doc_plugin_type (MooPlugin   *plugin,
                                 GType        type)
@@ -1117,6 +1099,9 @@ moo_plugin_set_doc_plugin_type (MooPlugin   *plugin,
 }
 
 
+/**
+ * moo_plugin_set_win_plugin_type:
+ */
 void
 moo_plugin_set_win_plugin_type (MooPlugin *plugin,
                                 GType      type)
@@ -1127,12 +1112,19 @@ moo_plugin_set_win_plugin_type (MooPlugin *plugin,
 }
 
 
+/**
+ * moo_plugin_info_new: (constructor-of MooPluginInfo)
+ *
+ * @name: (type const-utf8)
+ * @description: (type const-utf8) (allow-none) (default NULL)
+ * @author: (type const-utf8) (allow-none) (default NULL)
+ * @version: (type const-utf8) (allow-none) (default NULL)
+ */
 MooPluginInfo *
 moo_plugin_info_new (const char     *name,
                      const char     *description,
                      const char     *author,
-                     const char     *version,
-                     const char     *langs)
+                     const char     *version)
 {
     MooPluginInfo *info;
 
@@ -1140,10 +1132,9 @@ moo_plugin_info_new (const char     *name,
 
     info = g_new0 (MooPluginInfo, 1);
     info->name = g_strdup (name);
-    info->description = description ? g_strdup (description) : g_strdup ("");
-    info->author = author ? g_strdup (author) : g_strdup ("");
-    info->version = version ? g_strdup (version) : g_strdup ("");
-    info->langs = g_strdup (langs);
+    info->description = g_strdup (MOO_NZS (description));
+    info->author = g_strdup (MOO_NZS (author));
+    info->version = g_strdup (MOO_NZS (version));
 
     return info;
 }
@@ -1154,7 +1145,7 @@ moo_plugin_info_copy (MooPluginInfo *info)
 {
     g_return_val_if_fail (info != NULL, NULL);
     return moo_plugin_info_new (info->name, info->description,
-                                info->author, info->version, info->langs);
+                                info->author, info->version);
 }
 
 
@@ -1167,12 +1158,17 @@ moo_plugin_info_free (MooPluginInfo *info)
         g_free (info->description);
         g_free (info->author);
         g_free (info->version);
-        g_free (info->langs);
         g_free (info);
     }
 }
 
 
+/**
+ * moo_plugin_params_new: (constructor-of MooPluginParams)
+ *
+ * @enabled: (default TRUE)
+ * @visible: (default TRUE)
+ */
 MooPluginParams *
 moo_plugin_params_new (gboolean enabled,
                        gboolean visible)
@@ -1199,34 +1195,6 @@ moo_plugin_params_free (MooPluginParams *params)
 }
 
 
-GType
-moo_plugin_info_get_type (void)
-{
-    static GType type;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooPluginInfo",
-                                             (GBoxedCopyFunc) moo_plugin_info_copy,
-                                             (GBoxedFreeFunc) moo_plugin_info_free);
-
-    return type;
-}
-
-
-GType
-moo_plugin_params_get_type (void)
-{
-    static GType type;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooPluginParams",
-                                             (GBoxedCopyFunc) moo_plugin_params_copy,
-                                             (GBoxedFreeFunc) moo_plugin_params_free);
-
-    return type;
-}
-
-
 /***************************************************************************/
 /* Preferences dialog
  */
@@ -1240,14 +1208,12 @@ enum {
 
 
 static void
-selection_changed (GtkTreeSelection   *selection,
-                   MooPrefsDialogPage *page)
+selection_changed (GtkTreeSelection *selection,
+                   PrefsPageXml     *gxml)
 {
     MooPlugin *plugin = NULL;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GtkWidget *info;
-    GtkLabel *version, *author, *description;
 
     if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
@@ -1257,24 +1223,19 @@ selection_changed (GtkTreeSelection   *selection,
         g_free (id);
     }
 
-    info = moo_glade_xml_get_widget (page->xml, "info");
-    author = moo_glade_xml_get_widget (page->xml, "author");
-    version = moo_glade_xml_get_widget (page->xml, "version");
-    description = moo_glade_xml_get_widget (page->xml, "description");
-
-    gtk_widget_set_sensitive (info, plugin != NULL);
+    gtk_widget_set_sensitive (GTK_WIDGET (gxml->info), plugin != NULL);
 
     if (plugin)
     {
-        gtk_label_set_text (author, moo_plugin_author (plugin));
-        gtk_label_set_text (version, moo_plugin_version (plugin));
-        gtk_label_set_text (description, moo_plugin_description (plugin));
+        gtk_label_set_text (gxml->author, moo_plugin_author (plugin));
+        gtk_label_set_text (gxml->version, moo_plugin_version (plugin));
+        gtk_label_set_text (gxml->description, moo_plugin_description (plugin));
     }
     else
     {
-        gtk_label_set_text (author, "");
-        gtk_label_set_text (version, "");
-        gtk_label_set_text (description, "");
+        gtk_label_set_text (gxml->author, "");
+        gtk_label_set_text (gxml->version, "");
+        gtk_label_set_text (gxml->description, "");
     }
 }
 
@@ -1325,18 +1286,15 @@ sync_pages (MooPrefsDialog *dialog)
 
                 if (plugin_page)
                 {
-                    if (!MOO_IS_PREFS_DIALOG_PAGE (plugin_page))
+                    if (!MOO_IS_PREFS_PAGE (plugin_page))
                     {
-                        g_critical ("%s: oops", G_STRLOC);
+                        g_critical ("oops");
                     }
                     else
                     {
-                        MOO_PREFS_DIALOG_PAGE (plugin_page)->auto_apply = FALSE;
-
                         g_object_set_data_full (G_OBJECT (plugin_page), "moo-plugin-id",
                                                 g_strdup (moo_plugin_id (plugin)),
                                                 g_free);
-
                         plugin_pages = g_slist_append (plugin_pages, plugin_page);
                         moo_prefs_dialog_insert_page (dialog, plugin_page, -1);
                     }
@@ -1359,16 +1317,14 @@ sync_pages (MooPrefsDialog *dialog)
 
 
 static void
-prefs_init (MooPrefsDialog      *dialog,
-            MooPrefsDialogPage  *page)
+prefs_page_init (MooPrefsPage *page,
+                 PrefsPageXml *gxml)
 {
-    GtkTreeView *treeview;
     GtkListStore *store;
     GtkTreeModel *model;
     GSList *l, *plugins;
 
-    treeview = moo_glade_xml_get_widget (page->xml, "treeview");
-    model = gtk_tree_view_get_model (treeview);
+    model = gtk_tree_view_get_model (gxml->treeview);
     store = GTK_LIST_STORE (model);
 
     gtk_list_store_clear (store);
@@ -1390,24 +1346,24 @@ prefs_init (MooPrefsDialog      *dialog,
         }
     }
 
-    selection_changed (gtk_tree_view_get_selection (treeview), page);
+    selection_changed (gtk_tree_view_get_selection (gxml->treeview), gxml);
 
     g_slist_free (plugins);
-    sync_pages (dialog);
+    sync_pages (g_object_get_data (G_OBJECT (page), "moo-plugin-prefs-dialog"));
 }
 
 
 static void
-prefs_apply (MooPrefsDialog      *dialog,
-             MooPrefsDialogPage  *page)
+prefs_page_apply (MooPrefsPage *page,
+                  PrefsPageXml *gxml)
 {
-    GtkTreeView *treeview;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GSList *plugin_pages;
+    MooPrefsDialog *dialog;
 
-    treeview = moo_glade_xml_get_widget (page->xml, "treeview");
-    model = gtk_tree_view_get_model (treeview);
+    dialog = g_object_get_data (G_OBJECT (page), "moo-plugin-prefs-dialog");
+
+    model = gtk_tree_view_get_model (gxml->treeview);
 
     if (gtk_tree_model_get_iter_first (model, &iter)) do
     {
@@ -1436,15 +1392,6 @@ prefs_apply (MooPrefsDialog      *dialog,
     while (gtk_tree_model_iter_next (model, &iter));
 
     sync_pages (dialog);
-
-    plugin_pages = g_object_get_data (G_OBJECT (dialog),
-                                      "moo-plugin-prefs-pages");
-
-    while (plugin_pages)
-    {
-        g_signal_emit_by_name (plugin_pages->data, "apply");
-        plugin_pages = plugin_pages->next;
-    }
 }
 
 
@@ -1467,51 +1414,50 @@ enable_toggled (GtkCellRendererToggle *cell,
 void
 moo_plugin_attach_prefs (GtkWidget *dialog)
 {
-    MooPrefsDialogPage *page;
-    MooGladeXML *xml;
-    GtkTreeView *treeview;
+    GtkWidget *page;
     GtkCellRenderer *cell;
     GtkListStore *store;
     GtkTreeSelection *selection;
+    PrefsPageXml *gxml;
 
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
 
-    page = moo_prefs_dialog_page_new_from_xml ("Plugins", MOO_STOCK_PLUGINS,
-                                               NULL, moopluginprefs_glade_xml,
-                                               "page", MOO_PLUGIN_PREFS_ROOT);
-    g_return_if_fail (page != NULL);
+    page = moo_prefs_page_new (C_("Prefs page label", "Plugins"), MOO_STOCK_PLUGINS);
+    gxml = prefs_page_xml_new_with_root (page);
+    g_object_set_data (G_OBJECT (page), "moo-plugin-prefs-xml", gxml);
+    g_object_set_data (G_OBJECT (page), "moo-plugin-prefs-dialog", dialog);
+    g_signal_connect (page, "init", G_CALLBACK (prefs_page_init), gxml);
+    g_signal_connect (page, "apply", G_CALLBACK (prefs_page_apply), gxml);
 
     moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_PLUGINS);
 
-    xml = page->xml;
-
-    treeview = moo_glade_xml_get_widget (xml, "treeview");
-
-    selection = gtk_tree_view_get_selection (treeview);
+    selection = gtk_tree_view_get_selection (gxml->treeview);
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-    g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), page);
+    g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), gxml);
 
     store = gtk_list_store_new (N_COLUMNS,
                                 G_TYPE_BOOLEAN,
                                 G_TYPE_STRING,
                                 G_TYPE_STRING);
-    gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
+    gtk_tree_view_set_model (gxml->treeview, GTK_TREE_MODEL (store));
     g_object_unref (store);
 
     cell = gtk_cell_renderer_toggle_new ();
     g_object_set (cell, "activatable", TRUE, NULL);
     g_signal_connect (cell, "toggled", G_CALLBACK (enable_toggled), store);
-    gtk_tree_view_insert_column_with_attributes (treeview, 0, "Enabled", cell,
+    /* Column label on Plugins prefs page */
+    gtk_tree_view_insert_column_with_attributes (gxml->treeview, 0,
+                                                 C_("plugin-prefs-column", "Enabled"), cell,
                                                  "active", COLUMN_ENABLED, NULL);
 
     cell = gtk_cell_renderer_text_new ();
-    gtk_tree_view_insert_column_with_attributes (treeview, 1, "Plugin", cell,
+    /* Column label on Plugins prefs page */
+    gtk_tree_view_insert_column_with_attributes (gxml->treeview, 1,
+                                                 C_("plugin-prefs-column", "Plugin"), cell,
                                                  "text", COLUMN_PLUGIN_NAME, NULL);
 
-    g_signal_connect (dialog, "init", G_CALLBACK (prefs_init), page);
-    g_signal_connect (dialog, "apply", G_CALLBACK (prefs_apply), page);
-
     moo_prefs_dialog_append_page (MOO_PREFS_DIALOG (dialog), GTK_WIDGET (page));
+    prefs_page_init (MOO_PREFS_PAGE (page), gxml);
 }
 
 
@@ -1648,7 +1594,7 @@ moo_plugin_call_method_valist (gpointer        plugin,
 
         if (error)
         {
-            g_warning ("%s: %s", G_STRLOC, error);
+            g_warning ("%s", error);
             g_free (error);
 
             while (i--)
@@ -1689,7 +1635,7 @@ moo_plugin_call_method_valist (gpointer        plugin,
         }
         else
         {
-            g_warning ("%s: %s", G_STRLOC, error);
+            g_warning ("%s", error);
             g_free (error);
         }
     }
