@@ -16,10 +16,6 @@
 #ifndef MOO_TEXT_BTREE_H
 #define MOO_TEXT_BTREE_H
 
-#ifndef MOOEDIT_COMPILATION
-#error "This file may not be included"
-#endif
-
 #include <glib.h>
 
 G_BEGIN_DECLS
