@@ -1,18 +1,20 @@
 /*
  *   moofileentry.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* XXX use MooCombo */
 
-#define MOO_FILE_VIEW_COMPILATION
 #include "moofileentry.h"
 #include "moofilesystem.h"
 #include "moofoldermodel.h"
@@ -22,6 +24,7 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/mooaccel.h"
+#include "mooutils/moocompat.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
@@ -166,7 +169,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "enable-completion",
                                              "enable-completion",
                                              TRUE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_DIRECTORIES_ONLY,
@@ -174,7 +177,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "directories-only",
                                              "directories-only",
                                              FALSE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SHOW_HIDDEN,
@@ -182,7 +185,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "show-hidden",
                                              "show-hidden",
                                              TRUE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENTRY,
@@ -190,7 +193,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "entry",
                                              "entry",
                                              GTK_TYPE_ENTRY,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_CASE_SENSITIVE,
@@ -198,7 +201,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "case-sensitive",
                                              "case-sensitive",
                                              CASE_SENSITIVE_DEFAULT,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_CURRENT_DIR,
@@ -206,7 +209,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "current-dir",
                                              "current-dir",
                                              NULL,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_FILE_SYSTEM,
@@ -214,7 +217,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "file-system",
                                              "file-system",
                                              MOO_TYPE_FILE_SYSTEM,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     signals[FINISHED] =
             _moo_signal_new_cb ("finished",
@@ -242,19 +245,7 @@ _moo_file_entry_completion_init (MooFileEntryCompletion *cmpl)
 
     cmpl->priv->case_sensitive = CASE_SENSITIVE_DEFAULT;
     cmpl->priv->visible_func = (MooFileVisibleFunc) completion_default_visible_func;
-
-    if (cmpl->priv->case_sensitive)
-    {
-        cmpl->priv->text_funcs.strcmp_func = strcmp_func;
-        cmpl->priv->text_funcs.strncmp_func = strncmp_func;
-        cmpl->priv->text_funcs.normalize_func = normalize_func;
-    }
-    else
-    {
-        cmpl->priv->text_funcs.strcmp_func = case_strcmp_func;
-        cmpl->priv->text_funcs.strncmp_func = case_strncmp_func;
-        cmpl->priv->text_funcs.normalize_func = case_normalize_func;
-    }
+    set_text_funcs(&cmpl->priv->text_funcs, cmpl->priv->case_sensitive);
 
     model = _moo_folder_model_new (NULL);
     cmpl->priv->model = _moo_folder_filter_new (MOO_FOLDER_MODEL (model));
@@ -538,8 +529,8 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
                                       path, &dirname, &display_dirname,
                                       &display_basename, &error))
     {
-        _moo_message ("%s: could not parse path '%s'", G_STRLOC, path);
-        _moo_message ("%s: %s", G_STRLOC, error->message);
+        _moo_message ("could not parse path '%s'", path);
+        _moo_message ("%s", moo_error_message (error));
         goto out;
     }
 
@@ -552,7 +543,7 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
     /* this was needed in the widget */
     if (text_len <= cmpl->priv->display_basename_len)
     {
-        g_warning ("%s: something wrong", G_STRLOC);
+        g_warning ("something is wrong");
         goto out;
     }
 #endif
@@ -567,8 +558,8 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
                                               &error);
         if (!folder)
         {
-            _moo_message ("%s: could not get folder '%s'", G_STRLOC, dirname);
-            _moo_message ("%s: %s", G_STRLOC, error->message);
+            _moo_message ("could not get folder '%s'", dirname);
+            _moo_message ("%s", moo_error_message (error));
             g_error_free (error);
             goto out;
         }
@@ -642,7 +633,7 @@ completion_tab_key (MooFileEntryCompletion *cmpl)
 
         if (!prefix)
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
         }
         else if (prefix->len > cmpl->priv->display_basename_len)
         {
@@ -709,7 +700,7 @@ completion_popup_tab_key_press (MooFileEntryCompletion *cmpl)
 
         if (!prefix)
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
             return FALSE;
         }
 
@@ -745,20 +736,7 @@ completion_set_case_sensitive (MooFileEntryCompletion *cmpl,
     if (case_sensitive != cmpl->priv->case_sensitive)
     {
         cmpl->priv->case_sensitive = case_sensitive;
-
-        if (case_sensitive)
-        {
-            cmpl->priv->text_funcs.strcmp_func = strcmp_func;
-            cmpl->priv->text_funcs.strncmp_func = strncmp_func;
-            cmpl->priv->text_funcs.normalize_func = normalize_func;
-        }
-        else
-        {
-            cmpl->priv->text_funcs.strcmp_func = case_strcmp_func;
-            cmpl->priv->text_funcs.strncmp_func = case_strncmp_func;
-            cmpl->priv->text_funcs.normalize_func = case_normalize_func;
-        }
-
+        set_text_funcs (&cmpl->priv->text_funcs, case_sensitive);
         g_object_notify (G_OBJECT (cmpl), "case-sensitive");
     }
 }
@@ -785,7 +763,7 @@ completion_popup (MooFileEntryCompletion *cmpl)
 
     gtk_widget_realize (cmpl->priv->popup);
 
-    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (cmpl->priv->treeview), GTK_CAN_FOCUS);
+    GTK_WIDGET_SET_CAN_FOCUS (cmpl->priv->treeview);
 
     if (GTK_WINDOW (window)->group)
         gtk_window_group_add_window (GTK_WINDOW (window)->group,
@@ -1167,7 +1145,7 @@ folder_contents_changed (MooFileEntryCompletion *cmpl)
 {
     if (!cmpl->priv->resize_popup_idle)
         cmpl->priv->resize_popup_idle =
-            moo_idle_add ((GSourceFunc) resize_popup_idle, cmpl);
+            gdk_threads_add_idle ((GSourceFunc) resize_popup_idle, cmpl);
 }
 
 static void
@@ -1315,12 +1293,13 @@ completion_visible_func (GtkTreeModel           *model,
     }
     else if (cmpl->priv->visible_func (file, cmpl->priv->visible_func_data))
     {
-        visible = !cmpl->priv->text_funcs.strncmp_func (cmpl->priv->display_basename, file,
-                                                        cmpl->priv->display_basename_len);
+        visible = cmpl->priv->text_funcs.file_has_prefix (file,
+                                                          cmpl->priv->display_basename,
+                                                          cmpl->priv->display_basename_len);
     }
 
     _moo_file_unref (file);
-    return visible ? TRUE : FALSE; /* #314335 */
+    return visible;
 }
 
 
@@ -1631,7 +1610,7 @@ _moo_file_entry_class_init (MooFileEntryClass *klass)
 static void
 _moo_file_entry_init (G_GNUC_UNUSED MooFileEntry *entry)
 {
-    entry->completion = g_object_new (MOO_TYPE_FILE_ENTRY_COMPLETION, NULL);
+    entry->completion = MOO_FILE_ENTRY_COMPLETION (g_object_new (MOO_TYPE_FILE_ENTRY_COMPLETION, (const char*) NULL));
     entry->completion->priv->managed = TRUE;
     _moo_file_entry_completion_set_entry (entry->completion, GTK_ENTRY (entry));
 }
@@ -1689,10 +1668,10 @@ moo_file_entry_get_property (GObject        *object,
 }
 
 
-GtkWidget*
+GtkWidget *
 _moo_file_entry_new (void)
 {
-    return g_object_new (MOO_TYPE_FILE_ENTRY, NULL);
+    return GTK_WIDGET (g_object_new (MOO_TYPE_FILE_ENTRY, (const char*) NULL));
 }
 
 
