@@ -1,19 +1,18 @@
 /*
  *   moofoldermodel-private.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef MOO_FILE_VIEW_COMPILATION
-#error "This file may not be included"
-#endif
-
 #ifndef MOO_FOLDER_MODEL_PRIVATE_H
 #define MOO_FOLDER_MODEL_PRIVATE_H
 
@@ -90,7 +89,7 @@ static void      _hash_table_remove     (FileList       *flist,
                                          MooFile        *file);
 
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
 #if 0
 #define DEFINE_CHECK_FILE_LIST_INTEGRITY
 static void CHECK_FILE_LIST_INTEGRITY (FileList *flist)
@@ -114,7 +113,7 @@ static void CHECK_FILE_LIST_INTEGRITY (FileList *flist)
     }
 }
 #endif
-#endif /* MOO_DEBUG_ENABLED */
+#endif /* MOO_DEBUG */
 
 #ifndef DEFINE_CHECK_FILE_LIST_INTEGRITY
 #define CHECK_FILE_LIST_INTEGRITY(flist)
