@@ -1,7 +1,7 @@
 /*
  *   moolua-tests.h
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -20,7 +20,7 @@
 
 G_BEGIN_DECLS
 
-void    moo_test_lua                (void);
+void    moo_test_lua    (MooTestOptions opts);
 
 G_END_DECLS
 
