@@ -24,7 +24,7 @@ from moo import D_
 try:
     import vte
 except ImportError:
-    moo.cancel_plugin_loading()
+    moo._cancel_plugin_loading()
 
 TERMINAL_PLUGIN_ID = "Terminal"
 COLOR_SCHEME_KEY = 'Plugins/Terminal/color_scheme'
@@ -60,8 +60,8 @@ class Terminal(vte.Terminal):
             moo.prefs_set_string(COLOR_SCHEME_KEY, None)
 
     def font_item_activated(self, *whatever):
-        dlg = moo.FontSelectionDialog(D_("Pick a Font", "gtk20"))
-        dlg.fontsel.set_property('monospace', True)
+        dlg = gtk.FontSelectionDialog(D_("Pick a Font", "gtk20"))
+        # dlg.fontsel.set_property('monospace', True)
         old_font = self.get_font()
         if old_font:
             dlg.set_font_name(old_font.to_string())
