@@ -1,18 +1,20 @@
 /*
  *   mooaccelprefs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooaccelprefs.h"
 #include "mooutils/mooaccel.h"
-#include "glade/mooaccelprefs-glade.h"
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/mooaccelbutton.h"
 #include "mooutils/mooglade.h"
@@ -22,37 +24,35 @@
 #include "mooutils/mooaction-private.h"
 #include "mooutils/mooactiongroup.h"
 #include "mooutils/mootype-macros.h"
+#include "mooutils/moohelp.h"
+#include "mooutils/moocompat.h"
+#include "mooutils/mooaccelprefs-gxml.h"
+#include "moo-help-sections.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
 
 typedef struct {
-    MooPrefsDialogPage base;
+    MooPrefsPage base;
 
     GtkAction *current_action;
     GtkTreeRowReference *current_row;
 
-    GtkWidget *shortcut_frame;
-    GtkTreeView *treeview;
+    AccelPrefsPageXml *gxml;
     GtkTreeSelection *selection;
     GtkTreeStore *store;
-    GtkToggleButton *shortcut_default;
-    GtkToggleButton *shortcut_none;
-    GtkToggleButton *shortcut_custom;
-    MooAccelButton *shortcut;
-    GtkLabel *default_label;
 
     GHashTable *changed;    /* Gtkction* -> Shortcut* */
     GPtrArray *actions;     /* GtkActionGroup* */
     GHashTable *groups;     /* char* -> GtkTreeRowReference* */
 } MooAccelPrefsPage;
 
-typedef MooPrefsDialogPageClass MooAccelPrefsPageClass;
+typedef MooPrefsPageClass MooAccelPrefsPageClass;
 
 #define MOO_TYPE_ACCEL_PREFS_PAGE    (_moo_accel_prefs_page_get_type ())
 #define MOO_ACCEL_PREFS_PAGE(object) (G_TYPE_CHECK_INSTANCE_CAST ((object), MOO_TYPE_ACCEL_PREFS_PAGE, MooAccelPrefsPage))
-MOO_DEFINE_TYPE_STATIC (MooAccelPrefsPage, _moo_accel_prefs_page, MOO_TYPE_PREFS_DIALOG_PAGE)
+MOO_DEFINE_TYPE_STATIC (MooAccelPrefsPage, _moo_accel_prefs_page, MOO_TYPE_PREFS_PAGE)
 
 
 typedef enum {
@@ -71,20 +71,25 @@ enum {
     COLUMN_ACTION_NAME,
     COLUMN_ACTION,
     COLUMN_ACCEL,
+    COLUMN_GLOBAL,
     N_COLUMNS
 };
 
 
-static void moo_accel_prefs_page_init   (MooPrefsDialogPage *page);
-static void moo_accel_prefs_page_apply  (MooPrefsDialogPage *page);
+static void moo_accel_prefs_page_init   (MooPrefsPage       *page);
+static void moo_accel_prefs_page_apply  (MooPrefsPage       *page);
 static void tree_selection_changed      (MooAccelPrefsPage  *page);
 static void accel_set                   (MooAccelPrefsPage  *page);
 static void shortcut_none_toggled       (MooAccelPrefsPage  *page);
 static void shortcut_default_toggled    (MooAccelPrefsPage  *page);
 static void shortcut_custom_toggled     (MooAccelPrefsPage  *page);
 
-
-
+static void global_cell_data_func       (GtkTreeViewColumn  *column,
+                                         GtkCellRenderer    *cell,
+                                         GtkTreeModel       *model,
+                                         GtkTreeIter        *iter);
+static void global_cell_toggled         (GtkTreeStore       *store,
+                                         char               *path_string);
 
 static Shortcut *
 shortcut_new (ChoiceType  choice,
@@ -121,62 +126,111 @@ static void
 _moo_accel_prefs_page_class_init (MooAccelPrefsPageClass *klass)
 {
     G_OBJECT_CLASS (klass)->finalize = _moo_accel_prefs_page_finalize;
-    MOO_PREFS_DIALOG_PAGE_CLASS (klass)->init = moo_accel_prefs_page_init;
-    MOO_PREFS_DIALOG_PAGE_CLASS (klass)->apply = moo_accel_prefs_page_apply;
+    MOO_PREFS_PAGE_CLASS (klass)->init = moo_accel_prefs_page_init;
+    MOO_PREFS_PAGE_CLASS (klass)->apply = moo_accel_prefs_page_apply;
 }
 
 
 static void
 row_activated (MooAccelPrefsPage *page)
 {
-    if (GTK_WIDGET_IS_SENSITIVE (page->shortcut))
-        gtk_button_clicked (GTK_BUTTON (page->shortcut));
+    if (GTK_WIDGET_IS_SENSITIVE (page->gxml->shortcut))
+        gtk_button_clicked (GTK_BUTTON (page->gxml->shortcut));
 }
 
 
 static void
 block_accel_set (MooAccelPrefsPage *page)
 {
-    g_signal_handlers_block_matched (page->shortcut, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_block_matched (page->gxml->shortcut, G_SIGNAL_MATCH_FUNC,
                                      0, 0, 0, (gpointer) accel_set, 0);
 }
 
 static void
 unblock_accel_set (MooAccelPrefsPage *page)
 {
-    g_signal_handlers_unblock_matched (page->shortcut, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_unblock_matched (page->gxml->shortcut, G_SIGNAL_MATCH_FUNC,
                                        0, 0, 0, (gpointer) accel_set, 0);
 }
 
 static void
 block_radio (MooAccelPrefsPage *page)
 {
-    g_signal_handlers_block_matched (page->shortcut_none, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_block_matched (page->gxml->shortcut_none, G_SIGNAL_MATCH_FUNC,
                                      0, 0, 0, (gpointer) shortcut_none_toggled, 0);
-    g_signal_handlers_block_matched (page->shortcut_default, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_block_matched (page->gxml->shortcut_default, G_SIGNAL_MATCH_FUNC,
                                      0, 0, 0, (gpointer) shortcut_default_toggled, 0);
-    g_signal_handlers_block_matched (page->shortcut_custom, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_block_matched (page->gxml->shortcut_custom, G_SIGNAL_MATCH_FUNC,
                                      0, 0, 0, (gpointer) shortcut_custom_toggled, 0);
 }
 
 static void
 unblock_radio (MooAccelPrefsPage *page)
 {
-    g_signal_handlers_unblock_matched (page->shortcut_none, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_unblock_matched (page->gxml->shortcut_none, G_SIGNAL_MATCH_FUNC,
                                        0, 0, 0, (gpointer) shortcut_none_toggled, 0);
-    g_signal_handlers_unblock_matched (page->shortcut_default, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_unblock_matched (page->gxml->shortcut_default, G_SIGNAL_MATCH_FUNC,
                                        0, 0, 0, (gpointer) shortcut_default_toggled, 0);
-    g_signal_handlers_unblock_matched (page->shortcut_custom, G_SIGNAL_MATCH_FUNC,
+    g_signal_handlers_unblock_matched (page->gxml->shortcut_custom, G_SIGNAL_MATCH_FUNC,
                                        0, 0, 0, (gpointer) shortcut_custom_toggled, 0);
 }
 
 
+static void
+global_cell_data_func (G_GNUC_UNUSED GtkTreeViewColumn *column,
+                       GtkCellRenderer    *cell,
+                       GtkTreeModel       *model,
+                       GtkTreeIter        *iter)
+{
+    GtkAction *action = NULL;
+    char *accel = NULL;
+
+    gtk_tree_model_get (model, iter,
+                        COLUMN_ACTION, &action,
+                        COLUMN_ACCEL, &accel,
+                        -1);
+
+    if (action)
+        g_object_set (cell,
+                      "visible", TRUE,
+                      "sensitive", accel && accel[0],
+                      "activatable", accel && accel[0],
+                      NULL);
+    else
+        g_object_set (cell,
+                      "visible", FALSE,
+                      NULL);
+
+    if (action)
+        g_object_unref (action);
+    g_free (accel);
+}
+
+static void
+global_cell_toggled (GtkTreeStore *store,
+                     char         *path_string)
+{
+    GtkTreePath *path;
+    GtkTreeIter iter;
+
+    path = gtk_tree_path_new_from_string (path_string);
+
+    if (gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path))
+    {
+        gboolean value;
+        gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COLUMN_GLOBAL, &value, -1);
+        gtk_tree_store_set (store, &iter, COLUMN_GLOBAL, !value, -1);
+    }
+
+    gtk_tree_path_free (path);
+}
+
+
 static void
 _moo_accel_prefs_page_init (MooAccelPrefsPage *page)
 {
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
-    MooGladeXML *xml;
 
     page->changed = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL,
                                            (GDestroyNotify) shortcut_free);
@@ -185,66 +239,73 @@ _moo_accel_prefs_page_init (MooAccelPrefsPage *page)
                                           (GDestroyNotify) g_free,
                                           (GDestroyNotify) gtk_tree_row_reference_free);
 
-    xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
-    moo_glade_xml_fill_widget (xml, GTK_WIDGET (page),
-                               mooaccelprefs_glade_xml, -1,
-                               "page", NULL);
+    page->gxml = accel_prefs_page_xml_new_with_root (GTK_WIDGET (page));
     g_object_set (page, "label", "Shortcuts", "icon-stock-id", MOO_STOCK_KEYBOARD, NULL);
 
-    page->treeview = moo_glade_xml_get_widget (xml, "treeview");
-    gtk_tree_view_set_search_column (page->treeview, 0);
+    gtk_tree_view_set_search_column (page->gxml->treeview, 0);
 
-    g_signal_connect_swapped (page->treeview, "row-activated",
+    g_signal_connect_swapped (page->gxml->treeview, "row-activated",
                               G_CALLBACK (row_activated),
                               page);
 
-    page->shortcut_frame = moo_glade_xml_get_widget (xml, "shortcut_frame");
-    page->shortcut_default = moo_glade_xml_get_widget (xml, "shortcut_default");
-    page->shortcut_none = moo_glade_xml_get_widget (xml, "shortcut_none");
-    page->shortcut_custom = moo_glade_xml_get_widget (xml, "shortcut_custom");
-    page->shortcut = moo_glade_xml_get_widget (xml, "shortcut");
-    page->default_label = moo_glade_xml_get_widget (xml, "default_label");
-
-    g_object_unref (xml);
-
     page->store = gtk_tree_store_new (N_COLUMNS,
                                       G_TYPE_STRING,
                                       GTK_TYPE_ACTION,
-                                      G_TYPE_STRING);
-    gtk_tree_view_set_model (page->treeview, GTK_TREE_MODEL (page->store));
+                                      G_TYPE_STRING,
+                                      G_TYPE_BOOLEAN);
+    gtk_tree_view_set_model (page->gxml->treeview, GTK_TREE_MODEL (page->store));
     g_object_unref (page->store);
 
     renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes ("Action", renderer,
+    /* Column label in Configure Shortcuts dialog */
+    column = gtk_tree_view_column_new_with_attributes (C_("accel-editor-column", "Action"),
+                                                       renderer,
                                                        "text", COLUMN_ACTION_NAME,
                                                        NULL);
-    gtk_tree_view_append_column (page->treeview, column);
+    gtk_tree_view_append_column (page->gxml->treeview, column);
     gtk_tree_view_column_set_sort_column_id (column, COLUMN_ACTION_NAME);
 
     renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes ("Shortcut", renderer,
+    /* Column label in Configure Shortcuts dialog */
+    column = gtk_tree_view_column_new_with_attributes (C_("accel-editor-column", "Shortcut"),
+                                                       renderer,
                                                        "text", COLUMN_ACCEL,
                                                        NULL);
-    gtk_tree_view_append_column (page->treeview, column);
+    gtk_tree_view_append_column (page->gxml->treeview, column);
     gtk_tree_view_column_set_sort_column_id (column, COLUMN_ACCEL);
 
-    page->selection = gtk_tree_view_get_selection (page->treeview);
+    renderer = gtk_cell_renderer_toggle_new ();
+    /* Column label in Configure Shortcuts dialog */
+    column = gtk_tree_view_column_new_with_attributes (C_("accel-editor-column", "Global"),
+                                                       renderer,
+                                                       "active", COLUMN_GLOBAL,
+                                                       NULL);
+    gtk_tree_view_append_column (page->gxml->treeview, column);
+    gtk_tree_view_column_set_sort_column_id (column, COLUMN_GLOBAL);
+    gtk_tree_view_column_set_cell_data_func (column, renderer,
+                                             (GtkTreeCellDataFunc) global_cell_data_func,
+                                             NULL, NULL);
+    g_signal_connect_swapped (renderer, "toggled",
+                              G_CALLBACK (global_cell_toggled),
+                              page->store);
+
+    page->selection = gtk_tree_view_get_selection (page->gxml->treeview);
     gtk_tree_selection_set_mode (page->selection, GTK_SELECTION_SINGLE);
 
     g_signal_connect_swapped (page->selection, "changed",
                               G_CALLBACK (tree_selection_changed),
                               page);
 
-    g_signal_connect_swapped (page->shortcut, "accel-set",
+    g_signal_connect_swapped (page->gxml->shortcut, "accel-set",
                               G_CALLBACK (accel_set),
                               page);
-    g_signal_connect_swapped (page->shortcut_none, "toggled",
+    g_signal_connect_swapped (page->gxml->shortcut_none, "toggled",
                               G_CALLBACK (shortcut_none_toggled),
                               page);
-    g_signal_connect_swapped (page->shortcut_default, "toggled",
+    g_signal_connect_swapped (page->gxml->shortcut_default, "toggled",
                               G_CALLBACK (shortcut_default_toggled),
                               page);
-    g_signal_connect_swapped (page->shortcut_custom, "toggled",
+    g_signal_connect_swapped (page->gxml->shortcut_custom, "toggled",
                               G_CALLBACK (shortcut_custom_toggled),
                               page);
 }
@@ -284,12 +345,37 @@ apply_one (GtkAction *action,
     _moo_modify_accel (accel_path, new_accel);
 }
 
+static gboolean
+apply_global (GtkTreeModel *model,
+              G_GNUC_UNUSED GtkTreePath *path,
+              GtkTreeIter  *iter)
+{
+    GtkAction *action = NULL;
+    gboolean global = FALSE;
+
+    gtk_tree_model_get (model, iter,
+                        COLUMN_ACTION, &action,
+                        COLUMN_GLOBAL, &global,
+                        -1);
+
+    if (action)
+    {
+        _moo_accel_prefs_set_global (_moo_action_get_accel_path (action), global);
+        g_object_unref (action);
+    }
+
+    return FALSE;
+}
+
 static void
-moo_accel_prefs_page_apply (MooPrefsDialogPage *prefs_page)
+moo_accel_prefs_page_apply (MooPrefsPage *prefs_page)
 {
     MooAccelPrefsPage *page = MOO_ACCEL_PREFS_PAGE (prefs_page);
     g_hash_table_foreach (page->changed, (GHFunc) apply_one, NULL);
     g_hash_table_foreach_remove (page->changed, (GHRFunc) gtk_true, NULL);
+    gtk_tree_model_foreach (GTK_TREE_MODEL (page->store),
+                            (GtkTreeModelForeachFunc) apply_global,
+                            NULL);
 }
 
 
@@ -311,6 +397,7 @@ add_row (GtkActionGroup    *group,
     const char *name;
     GtkTreeIter iter;
     GtkTreeRowReference *row;
+    gboolean global;
 
     if (_moo_action_get_no_accel (action) || !_moo_action_get_accel_editable (action))
         return FALSE;
@@ -320,7 +407,7 @@ add_row (GtkActionGroup    *group,
     if (!group_name)
         group_name = "";
 
-    row = g_hash_table_lookup (page->groups, group_name);
+    row = (GtkTreeRowReference*) g_hash_table_lookup (page->groups, group_name);
 
     if (row)
     {
@@ -328,8 +415,7 @@ add_row (GtkActionGroup    *group,
         GtkTreePath *path = gtk_tree_row_reference_get_path (row);
         if (!path || !gtk_tree_model_get_iter (GTK_TREE_MODEL (page->store), &parent, path))
         {
-            g_critical ("%s: got invalid path for group %s",
-                        G_STRLOC, group_name);
+            g_critical ("got invalid path for group %s", group_name);
             gtk_tree_path_free (path);
             return FALSE;
         }
@@ -359,11 +445,13 @@ add_row (GtkActionGroup    *group,
 
     accel = get_accel_label_for_path (_moo_action_get_accel_path (action));
     name = _moo_action_get_display_name (action);
+    global = _moo_accel_prefs_get_global (gtk_action_get_accel_path (action));
 
     gtk_tree_store_set (page->store, &iter,
                         COLUMN_ACTION_NAME, name,
                         COLUMN_ACTION, action,
                         COLUMN_ACCEL, accel,
+                        COLUMN_GLOBAL, global,
                         -1);
     g_free (accel);
 
@@ -372,19 +460,19 @@ add_row (GtkActionGroup    *group,
 
 
 static void
-moo_accel_prefs_page_init (MooPrefsDialogPage *prefs_page)
+moo_accel_prefs_page_init (MooPrefsPage *prefs_page)
 {
     guint i;
     MooAccelPrefsPage *page = MOO_ACCEL_PREFS_PAGE (prefs_page);
 
     for (i = 0; i < page->actions->len; ++i)
     {
-        GtkActionGroup *group = g_ptr_array_index (page->actions, i);
+        GtkActionGroup *group = (GtkActionGroup*) g_ptr_array_index (page->actions, i);
         GList *list = gtk_action_group_list_actions (group);
 
         while (list)
         {
-            add_row (group, list->data, page);
+            add_row (group, GTK_ACTION (list->data), page);
             list = g_list_delete_link (list, list);
         }
     }
@@ -392,7 +480,7 @@ moo_accel_prefs_page_init (MooPrefsDialogPage *prefs_page)
     gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (page->store),
                                           COLUMN_ACTION_NAME,
                                           GTK_SORT_ASCENDING);
-    gtk_tree_view_expand_all (page->treeview);
+    gtk_tree_view_expand_all (page->gxml->treeview);
     tree_selection_changed (page);
 }
 
@@ -406,7 +494,7 @@ tree_selection_changed (MooAccelPrefsPage *page)
     GtkTreePath *path;
     char *default_label;
     const char *default_accel, *new_accel = "";
-    GtkToggleButton *new_button = NULL;
+    GtkRadioButton *new_button = NULL;
     Shortcut *shortcut;
 
     if (gtk_tree_selection_get_selected (page->selection, NULL, &iter))
@@ -423,8 +511,8 @@ tree_selection_changed (MooAccelPrefsPage *page)
 
     if (!selected_action)
     {
-        gtk_label_set_text (page->default_label, "");
-        gtk_widget_set_sensitive (page->shortcut_frame, FALSE);
+        gtk_label_set_text (page->gxml->default_label, "");
+        gtk_widget_set_sensitive (GTK_WIDGET (page->gxml->shortcut_frame), FALSE);
         return;
     }
 
@@ -432,37 +520,37 @@ tree_selection_changed (MooAccelPrefsPage *page)
     page->current_row = gtk_tree_row_reference_new (GTK_TREE_MODEL (page->store), path);
     gtk_tree_path_free (path);
 
-    gtk_widget_set_sensitive (page->shortcut_frame, TRUE);
+    gtk_widget_set_sensitive (GTK_WIDGET (page->gxml->shortcut_frame), TRUE);
 
     default_accel = _moo_get_default_accel (_moo_action_get_accel_path (action));
     default_label = _moo_get_accel_label (default_accel);
 
     if (!default_label || !default_label[0])
-        gtk_label_set_text (page->default_label, "None");
+        gtk_label_set_text (page->gxml->default_label, "None");
     else
-        gtk_label_set_text (page->default_label, default_label);
+        gtk_label_set_text (page->gxml->default_label, default_label);
 
     block_radio (page);
     block_accel_set (page);
 
-    shortcut = g_hash_table_lookup (page->changed, action);
+    shortcut = (Shortcut*) g_hash_table_lookup (page->changed, action);
 
     if (shortcut)
     {
         switch (shortcut->choice)
         {
             case NONE:
-                new_button = page->shortcut_none;
+                new_button = page->gxml->shortcut_none;
                 new_accel = NULL;
                 break;
 
             case DEFAULT:
-                new_button = page->shortcut_default;
+                new_button = page->gxml->shortcut_default;
                 new_accel = default_accel;
                 break;
 
             case CUSTOM:
-                new_button = page->shortcut_custom;
+                new_button = page->gxml->shortcut_custom;
                 new_accel = shortcut->accel;
                 break;
 
@@ -476,23 +564,23 @@ tree_selection_changed (MooAccelPrefsPage *page)
 
         if (!strcmp (accel, default_accel))
         {
-            new_button = page->shortcut_default;
+            new_button = page->gxml->shortcut_default;
             new_accel = default_accel;
         }
         else if (!accel[0])
         {
-            new_button = page->shortcut_none;
+            new_button = page->gxml->shortcut_none;
             new_accel = NULL;
         }
         else
         {
-            new_button = page->shortcut_custom;
+            new_button = page->gxml->shortcut_custom;
             new_accel = accel;
         }
     }
 
-    gtk_toggle_button_set_active (new_button, TRUE);
-    _moo_accel_button_set_accel (page->shortcut, new_accel);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (new_button), TRUE);
+    _moo_accel_button_set_accel (page->gxml->shortcut, new_accel);
 
     unblock_radio (page);
     unblock_accel_set (page);
@@ -513,20 +601,20 @@ accel_set (MooAccelPrefsPage *page)
     g_return_if_fail (page->current_row != NULL && page->current_action != NULL);
 
     block_radio (page);
-    gtk_toggle_button_set_active (page->shortcut_custom, TRUE);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->gxml->shortcut_custom), TRUE);
     unblock_radio (page);
 
     path = gtk_tree_row_reference_get_path (page->current_row);
     if (!path || !gtk_tree_model_get_iter (GTK_TREE_MODEL (page->store), &iter, path))
     {
-        g_critical ("%s: got invalid path", G_STRLOC);
+        g_critical ("got invalid path");
         gtk_tree_path_free (path);
         return;
     }
     gtk_tree_path_free (path);
 
-    accel = _moo_accel_button_get_accel (page->shortcut);
-    label = gtk_button_get_label (GTK_BUTTON (page->shortcut));
+    accel = _moo_accel_button_get_accel (page->gxml->shortcut);
+    label = gtk_button_get_label (GTK_BUTTON (page->gxml->shortcut));
     gtk_tree_store_set (page->store, &iter, COLUMN_ACCEL, label, -1);
 
     if (accel && accel[0])
@@ -548,13 +636,13 @@ shortcut_none_toggled (MooAccelPrefsPage *page)
 
     g_return_if_fail (page->current_row != NULL && page->current_action != NULL);
 
-    if (!gtk_toggle_button_get_active (page->shortcut_none))
+    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->gxml->shortcut_none)))
         return;
 
     path = gtk_tree_row_reference_get_path (page->current_row);
     if (!path || !gtk_tree_model_get_iter (GTK_TREE_MODEL (page->store), &iter, path))
     {
-        g_critical ("%s: got invalid path", G_STRLOC);
+        g_critical ("got invalid path");
         gtk_tree_path_free (path);
         return;
     }
@@ -565,7 +653,7 @@ shortcut_none_toggled (MooAccelPrefsPage *page)
                          page->current_action,
                          shortcut_new (NONE, ""));
     block_accel_set (page);
-    _moo_accel_button_set_accel (page->shortcut, "");
+    _moo_accel_button_set_accel (page->gxml->shortcut, "");
     unblock_accel_set (page);
 }
 
@@ -580,21 +668,21 @@ shortcut_default_toggled (MooAccelPrefsPage *page)
 
     g_return_if_fail (page->current_row != NULL && page->current_action != NULL);
 
-    if (!gtk_toggle_button_get_active (page->shortcut_default))
+    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->gxml->shortcut_default)))
         return;
 
     path = gtk_tree_row_reference_get_path (page->current_row);
 
     if (!path || !gtk_tree_model_get_iter (GTK_TREE_MODEL (page->store), &iter, path))
     {
-        g_critical ("%s: got invalid path", G_STRLOC);
+        g_critical ("got invalid path");
         gtk_tree_path_free (path);
         return;
     }
 
     gtk_tree_path_free (path);
 
-    current_shortcut = g_hash_table_lookup (page->changed, page->current_action);
+    current_shortcut = (Shortcut*) g_hash_table_lookup (page->changed, page->current_action);
 
     if (!current_shortcut)
     {
@@ -622,7 +710,7 @@ shortcut_default_toggled (MooAccelPrefsPage *page)
     current_shortcut->accel = g_strdup (default_accel);
 
     block_accel_set (page);
-    _moo_accel_button_set_accel (page->shortcut, default_accel);
+    _moo_accel_button_set_accel (page->gxml->shortcut, default_accel);
     unblock_accel_set (page);
 }
 
@@ -636,37 +724,37 @@ shortcut_custom_toggled (MooAccelPrefsPage *page)
 
     g_return_if_fail (page->current_row != NULL && page->current_action != NULL);
 
-    if (!gtk_toggle_button_get_active (page->shortcut_custom))
+    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->gxml->shortcut_custom)))
         return;
 
     path = gtk_tree_row_reference_get_path (page->current_row);
 
     if (!path || !gtk_tree_model_get_iter (GTK_TREE_MODEL (page->store), &iter, path))
     {
-        g_critical ("%s: got invalid path", G_STRLOC);
+        g_critical ("got invalid path");
         gtk_tree_path_free (path);
         return;
     }
 
     gtk_tree_path_free (path);
 
-    shortcut = g_hash_table_lookup (page->changed, page->current_action);
+    shortcut = (Shortcut*) g_hash_table_lookup (page->changed, page->current_action);
 
     if (shortcut)
     {
         block_accel_set (page);
-        _moo_accel_button_set_accel (page->shortcut, shortcut->accel);
+        _moo_accel_button_set_accel (page->gxml->shortcut, shortcut->accel);
         unblock_accel_set (page);
 
         gtk_tree_store_set (page->store, &iter, COLUMN_ACCEL,
-                            gtk_button_get_label (GTK_BUTTON (page->shortcut)),
+                            gtk_button_get_label (GTK_BUTTON (page->gxml->shortcut)),
                             -1);
 
         shortcut->choice = CUSTOM;
     }
     else
     {
-        gtk_button_clicked (GTK_BUTTON (page->shortcut));
+        gtk_button_clicked (GTK_BUTTON (page->gxml->shortcut));
     }
 }
 
@@ -694,7 +782,7 @@ _moo_accel_prefs_page_new (MooActionCollection *collection)
     const GSList *groups;
     MooAccelPrefsPage *page;
 
-    page = g_object_new (MOO_TYPE_ACCEL_PREFS_PAGE, NULL);
+    page = MOO_ACCEL_PREFS_PAGE (g_object_new (MOO_TYPE_ACCEL_PREFS_PAGE, (const char*) NULL));
     groups = moo_action_collection_get_groups (collection);
 
     while (groups)
@@ -707,54 +795,53 @@ _moo_accel_prefs_page_new (MooActionCollection *collection)
 }
 
 
-static GtkWidget*
+static GtkWidget *
 _moo_accel_prefs_dialog_new (MooActionCollection *collection)
 {
     MooAccelPrefsPage *page;
-    GtkWidget *dialog, *page_holder;
-    MooGladeXML *xml;
-
-    xml = moo_glade_xml_new_from_buf (mooaccelprefs_glade_xml, -1,
-                                      "dialog", GETTEXT_PACKAGE, NULL);
-    g_return_val_if_fail (xml != NULL, NULL);
-
-    dialog = moo_glade_xml_get_widget (xml, "dialog");
-
-    page = _moo_accel_prefs_page_new (collection);
-    gtk_widget_show (GTK_WIDGET (page));
-    page_holder = moo_glade_xml_get_widget (xml, "page_holder");
-    gtk_container_add (GTK_CONTAINER (page_holder), GTK_WIDGET (page));
-
-    g_object_unref (xml);
+    GtkWidget *dialog;
 
+    dialog = gtk_dialog_new_with_buttons (_("Configure Shortcuts"), NULL,
+                                          (GtkDialogFlags) 0,
+                                          GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
+                                          NULL);
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                              GTK_RESPONSE_OK,
                                              GTK_RESPONSE_CANCEL,
                                              -1);
+    gtk_window_set_default_size (GTK_WINDOW (dialog), -1, 400);
+
+    page = _moo_accel_prefs_page_new (collection);
+    gtk_widget_show (GTK_WIDGET (page));
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), GTK_WIDGET (page), TRUE, TRUE, 0);
+
+    moo_help_set_id (dialog, HELP_SECTION_PREFS_ACCELS);
+    moo_help_connect_keys (dialog);
 
     g_signal_connect_swapped (dialog, "response",
-                              G_CALLBACK (dialog_response), page);
+                              G_CALLBACK (dialog_response),
+                              page);
     g_signal_emit_by_name (page, "init", NULL);
 
     return dialog;
 }
 
 
-void
+gboolean
 _moo_accel_prefs_dialog_run (MooActionCollection *collection,
                              GtkWidget           *parent)
 {
-    GtkWidget *dialog = _moo_accel_prefs_dialog_new (collection);
+    GtkWidget *dialog;
+    int response;
 
+    dialog = _moo_accel_prefs_dialog_new (collection);
     moo_window_set_parent (dialog, parent);
 
-    while (TRUE)
-    {
-        int response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-        if (response != GTK_RESPONSE_REJECT)
-            break;
-    }
+    while ((response = gtk_dialog_run (GTK_DIALOG (dialog))) == GTK_RESPONSE_HELP)
+        moo_help_open (dialog);
 
     gtk_widget_destroy (dialog);
+    return response == GTK_RESPONSE_OK;
 }
