@@ -1,7 +1,7 @@
 /*
  *   mooapp/mooappinput.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -77,6 +77,7 @@ static InputChannel *input_channel_new      (const char     *appname,
                                              gboolean        may_fail);
 static void          input_channel_free     (InputChannel   *ch);
 static char         *input_channel_get_path (InputChannel   *ch);
+G_GNUC_UNUSED
 static const char   *input_channel_get_name (InputChannel   *ch);
 
 
@@ -101,28 +102,26 @@ moo_app_input_new (const char *name,
 {
     MooAppInput *ch;
     InputChannel *ich;
-    const char *appname = g_get_prgname ();
 
-    g_return_val_if_fail (appname != NULL, FALSE);
     g_return_val_if_fail (callback != NULL, NULL);
 
-    ch = moo_new0 (MooAppInput);
+    ch = g_slice_new0 (MooAppInput);
 
     ch->callback = callback;
     ch->callback_data = callback_data;
     ch->pipes = NULL;
-    ch->appname = g_strdup (appname);
+    ch->appname = g_strdup (MOO_PACKAGE_NAME);
 
-    if ((ich = input_channel_new (appname, _moo_get_pid_string (), FALSE)))
+    if ((ich = input_channel_new (ch->appname, _moo_get_pid_string (), FALSE)))
     {
         ch->pipes = g_slist_prepend (ch->pipes, ich);
         ch->main_path = input_channel_get_path (ich);
     }
 
-    if (name && (ich = input_channel_new (appname, name, FALSE)))
+    if (name && (ich = input_channel_new (ch->appname, name, FALSE)))
         ch->pipes = g_slist_prepend (ch->pipes, ich);
 
-    if (bind_default && (ich = input_channel_new (appname, MOO_APP_INPUT_NAME_DEFAULT, TRUE)))
+    if (bind_default && (ich = input_channel_new (ch->appname, MOO_APP_INPUT_NAME_DEFAULT, TRUE)))
         ch->pipes = g_slist_prepend (ch->pipes, ich);
 
     return ch;
@@ -149,7 +148,7 @@ moo_app_input_free (MooAppInput *ch)
 
     g_free (ch->main_path);
     g_free (ch->appname);
-    moo_free (MooAppInput, ch);
+    g_slice_free (MooAppInput, ch);
 }
 
 void
@@ -187,7 +186,7 @@ commit (GString **buffer)
 
     if (!(*buffer)->len)
     {
-        moo_dmsg ("%s: got empty command", G_STRLOC);
+        moo_dmsg ("got empty command");
         return;
     }
 
@@ -386,14 +385,12 @@ _moo_app_input_send_msg (const char *name,
     GDir *pipe_dir = NULL;
     char *pipe_dir_name;
     gboolean success = FALSE;
-    const char *appname = g_get_prgname ();
 
-    g_return_val_if_fail (appname != NULL, FALSE);
     g_return_val_if_fail (data != NULL, FALSE);
 
     moo_dmsg ("_moo_app_input_send_msg: sending data to %s", name ? name : "NONE");
 
-    pipe_dir_name = get_pipe_dir (appname);
+    pipe_dir_name = get_pipe_dir (MOO_PACKAGE_NAME);
     g_return_val_if_fail (pipe_dir_name != NULL, FALSE);
 
     if (name)
@@ -495,7 +492,7 @@ do_write (int         fd,
         {
             if (errno != EAGAIN && errno != EINTR)
             {
-                g_warning ("%s in write: %s", G_STRLOC, g_strerror (errno));
+                g_warning ("in write: %s", g_strerror (errno));
                 return FALSE;
             }
         }
@@ -568,7 +565,7 @@ connection_free (Connection *conn)
         close (conn->fd);
 
     g_string_free (conn->buffer, TRUE);
-    moo_free (Connection, conn);
+    g_slice_free (Connection, conn);
 }
 
 static void
@@ -631,23 +628,22 @@ read_input (G_GNUC_UNUSED GIOChannel *source,
     if (n <= 0)
     {
         if (n < 0)
-            moo_dmsg ("%s: %s", G_STRLOC, g_strerror (errno));
+            moo_dmsg ("%s", g_strerror (errno));
         else
-            moo_dmsg ("%s: EOF", G_STRLOC);
+            moo_dmsg ("EOF");
         goto remove;
     }
     else
     {
-        moo_dmsg ("%s: got bytes: '%s'", G_STRLOC, conn->buffer->str);
+        moo_dmsg ("got bytes: '%s'", conn->buffer->str);
     }
 
     if (do_commit)
         commit (&conn->buffer);
 
-    if (condition & (G_IO_ERR | G_IO_HUP))
+    if (!do_commit && (condition & (G_IO_ERR | G_IO_HUP)))
     {
-        moo_dmsg ("%s: %s", G_STRLOC,
-                  (condition & G_IO_ERR) ? "G_IO_ERR" : "G_IO_HUP");
+        moo_dmsg ("%s", (condition & G_IO_ERR) ? "G_IO_ERR" : "G_IO_HUP");
         goto remove;
     }
 
@@ -674,7 +670,7 @@ accept_connection (G_GNUC_UNUSED GIOChannel *source,
         return FALSE;
     }
 
-    conn = moo_new0 (Connection);
+    conn = g_slice_new0 (Connection);
     conn->ch = ch;
     conn->buffer = g_string_new_len (NULL, MAX_BUFFER_SIZE);
 
@@ -682,8 +678,8 @@ accept_connection (G_GNUC_UNUSED GIOChannel *source,
 
     if (conn->fd == -1)
     {
-        g_warning ("%s in accept: %s", G_STRLOC, g_strerror (errno));
-        moo_free (Connection, conn);
+        g_warning ("in accept: %s", g_strerror (errno));
+        g_slice_free (Connection, conn);
         return TRUE;
     }
 
@@ -691,7 +687,7 @@ accept_connection (G_GNUC_UNUSED GIOChannel *source,
                                  &conn->io, &conn->io_watch))
     {
         close (conn->fd);
-        moo_free (Connection, conn);
+        g_slice_free (Connection, conn);
         return TRUE;
     }
 
@@ -710,7 +706,7 @@ try_connect (const char *filename,
 
     if (strlen (filename) + 1 > sizeof addr.sun_path)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         return FALSE;
     }
 
@@ -720,7 +716,7 @@ try_connect (const char *filename,
 
     if (fd == -1)
     {
-        g_warning ("%s in socket for %s: %s", G_STRLOC, filename, g_strerror (errno));
+        g_warning ("in socket for %s: %s", filename, g_strerror (errno));
         return FALSE;
     }
 
@@ -752,14 +748,13 @@ input_channel_start (InputChannel *ch,
     if (try_connect (ch->path, NULL))
     {
         if (!may_fail)
-            g_warning ("%s: '%s' is already in use",
-                       G_STRLOC, ch->path);
+            g_warning ("'%s' is already in use", ch->path);
         return FALSE;
     }
 
     if (strlen (ch->path) + 1 > sizeof addr.sun_path)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         return FALSE;
     }
 
@@ -770,13 +765,13 @@ input_channel_start (InputChannel *ch,
 
     if ((ch->fd = socket (PF_UNIX, SOCK_STREAM, 0)) == -1)
     {
-        g_warning ("%s in socket for %s: %s", G_STRLOC, ch->path, g_strerror (errno));
+        g_warning ("in socket for %s: %s", ch->path, g_strerror (errno));
         return FALSE;
     }
 
     if (bind (ch->fd, (struct sockaddr*) &addr, sizeof addr) == -1)
     {
-        g_warning ("%s in bind for %s: %s", G_STRLOC, ch->path, g_strerror (errno));
+        g_warning ("in bind for %s: %s", ch->path, g_strerror (errno));
         close (ch->fd);
         ch->fd = -1;
         return FALSE;
@@ -784,7 +779,7 @@ input_channel_start (InputChannel *ch,
 
     if (listen (ch->fd, 5) == -1)
     {
-        g_warning ("%s in listen for %s: %s", G_STRLOC, ch->path, g_strerror (errno));
+        g_warning ("in listen for %s: %s", ch->path, g_strerror (errno));
         close (ch->fd);
     	ch->fd = -1;
     	return FALSE;
@@ -809,7 +804,7 @@ input_channel_new (const char *appname,
     g_return_val_if_fail (appname != NULL, NULL);
     g_return_val_if_fail (name != NULL, NULL);
 
-    ch = moo_new0 (InputChannel);
+    ch = g_slice_new0 (InputChannel);
 
     ch->name = g_strdup (name);
     ch->pipe_dir = get_pipe_dir (appname);
@@ -840,7 +835,7 @@ input_channel_free (InputChannel *ch)
         g_free (ch->pipe_dir);
     }
 
-    moo_free (InputChannel, ch);
+    g_slice_free (InputChannel, ch);
 }
 
 
@@ -977,8 +972,7 @@ listener_main (ListenerInfo *info)
             if (err != ERROR_PIPE_CONNECTED)
             {
                 char *msg = g_win32_error_message (err);
-                _moo_message_async ("%s: error in ConnectNamedPipe()", G_STRLOC);
-                _moo_message_async ("%s: %s", G_STRLOC, msg);
+                _moo_message_async ("%s: error in ConnectNamedPipe(): %s", G_STRLOC, msg);
                 CloseHandle (input);
                 g_free (msg);
                 break;
@@ -1072,7 +1066,7 @@ input_channel_new (const char *appname,
 {
     InputChannel *ch;
 
-    ch = moo_new0 (InputChannel);
+    ch = g_slice_new0 (InputChannel);
     ch->appname = g_strdup (appname);
     ch->name = g_strdup (name);
     ch->buffer = g_string_new_len (NULL, MAX_BUFFER_SIZE);
@@ -1096,7 +1090,7 @@ input_channel_free (InputChannel *ch)
     g_free (ch->pipe_name);
     g_free (ch->appname);
     g_free (ch->name);
-    moo_free (InputChannel, ch);
+    g_slice_free (InputChannel, ch);
 }
 
 static gboolean
@@ -1133,9 +1127,7 @@ _moo_app_input_send_msg (const char *name,
     char *pipe_name;
     HANDLE pipe_handle;
     gboolean result = FALSE;
-    const char *appname = g_get_prgname ();
 
-    g_return_val_if_fail (appname != NULL, FALSE);
     g_return_val_if_fail (data != NULL, FALSE);
 
     if (len < 0)
@@ -1147,7 +1139,7 @@ _moo_app_input_send_msg (const char *name,
     if (!name)
         name = "main";
 
-    pipe_name = get_pipe_name (appname, name);
+    pipe_name = get_pipe_name (MOO_PACKAGE_NAME, name);
 	/* XXX unicode */
     pipe_handle = CreateFileA (pipe_name, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
 
