@@ -107,6 +107,26 @@ _moo_noop_gtk_dialog_set_has_separator (G_GNUC_UNUSED GtkDialog *dialog,
 #endif /* gtk-2.22.0 && DISABLE_DEPRECATED */
 
 
+#if GTK_CHECK_VERSION(2,24,0) && defined(GTK_DISABLE_DEPRECATED)
+
+inline static void
+_moo_gdk_drawable_get_size (GdkDrawable *drawable,
+                            gint        *width,
+                            gint        *height)
+{
+    if (width)
+        *width = gdk_window_get_width (GDK_WINDOW (drawable));
+    if (height)
+        *height = gdk_window_get_height (GDK_WINDOW (drawable));
+}
+
+#define gdk_drawable_get_size _moo_gdk_drawable_get_size
+
+#else /* gtk-2.242.0 && DISABLE_DEPRECATED */
+
+#endif /* gtk-2.24.0 && DISABLE_DEPRECATED */
+
+
 #if GLIB_CHECK_VERSION(2,22,0) && defined(G_DISABLE_DEPRECATED)
 
 inline static void
