@@ -131,9 +131,12 @@ static int
 compare_encodings (Encoding **enc1,
                    Encoding **enc2)
 {
-    int result;
+    int result = 0;
 
-    result = g_utf8_collate ((*enc1)->subgroup_name, (*enc2)->subgroup_name);
+    if ((*enc1)->subgroup_name != NULL && (*enc2)->subgroup_name != NULL)
+        result = g_utf8_collate ((*enc1)->subgroup_name, (*enc2)->subgroup_name);
+    else
+        result = (*enc1)->subgroup_name < (*enc2)->subgroup_name;
 
     if (!result)
         result = g_utf8_collate ((*enc1)->short_display_name, (*enc2)->short_display_name);
@@ -142,7 +145,8 @@ compare_encodings (Encoding **enc1,
 }
 
 static void
-fill_encoding_group (EncodingGroup *group,
+fill_encoding_group (int            group_no,
+                     EncodingGroup *group,
                      GSList        *encodings)
 {
     guint i;
@@ -156,8 +160,9 @@ fill_encoding_group (EncodingGroup *group,
     for (i = 0; encodings != NULL; encodings = encodings->next, ++i)
         group->encodings[i] = (Encoding*) encodings->data;
 
-    qsort (group->encodings, group->n_encodings, sizeof (Encoding*),
-           (int(*)(const void *, const void *)) compare_encodings);
+    if (group_no != ENCODING_GROUP_UNICODE)
+        qsort (group->encodings, group->n_encodings, sizeof (Encoding*),
+               (int(*)(const void *, const void *)) compare_encodings);
 }
 
 static void
@@ -243,10 +248,19 @@ get_enc_mgr (void)
 
             enc = g_new0 (Encoding, 1);
             enc->name = moo_encodings_data[i].name;
-            enc->subgroup_name = g_strdup (_(moo_encodings_data[i].display_subgroup));
-            enc->short_display_name = moo_encodings_data[i].short_display_name;
-            enc->display_name = g_strdup_printf ("%s (%s)", enc->subgroup_name,
-                                                 enc->short_display_name);
+
+            if (moo_encodings_data[i].display_subgroup != NULL)
+            {
+                enc->short_display_name = moo_encodings_data[i].short_display_name;
+                enc->subgroup_name = g_strdup (_(moo_encodings_data[i].display_subgroup));
+                enc->display_name = g_strdup_printf ("%s (%s)", enc->subgroup_name,
+                                                     enc->short_display_name);
+            }
+            else
+            {
+                enc->short_display_name = Q_(moo_encodings_data[i].short_display_name);
+                enc->display_name = g_strdup (enc->short_display_name);
+            }
 
             enc_groups[moo_encodings_data[i].group] =
                 g_slist_prepend (enc_groups[moo_encodings_data[i].group], enc);
@@ -269,7 +283,7 @@ get_enc_mgr (void)
 
         for (i = 0; i < N_ENCODING_GROUPS; ++i)
         {
-            fill_encoding_group (&mgr->groups[i], enc_groups[i]);
+            fill_encoding_group (i, &mgr->groups[i], enc_groups[i]);
             g_slist_free (enc_groups[i]);
         }
 
@@ -714,7 +728,7 @@ setup_combo (GtkComboBox      *combo,
         gtk_tree_store_append (store, &iter, NULL);
 
     gtk_tree_store_append (store, &iter, NULL);
-    /* Translators: do not translate the part before | */
+    /* Translators: remove the part before and including | */
     gtk_tree_store_set (store, &iter, COLUMN_DISPLAY, Q_("Encodings combo submenu|Other"), -1);
 
     for (i = 0; i < enc_mgr->n_groups; ++i)
