@@ -24,10 +24,7 @@
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooeditops.h"
 #include "mooutils/mooi18n.h"
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtkseparatormenuitem.h>
-#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
