@@ -1,15 +1,22 @@
 /*
  *   moofiledialog.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooFileDialog: (parent GObject) (constructable) (moo.private 1)
+ **/
+
 #include "config.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooprefs.h"
@@ -18,12 +25,13 @@
 #include "mooutils/moohelp.h"
 #include "mooutils/moofiltermgr.h"
 #include "mooutils/mooutils-enums.h"
+#include "mooutils/mooi18n.h"
 #include "marshals.h"
 #include <gtk/gtk.h>
 #include <string.h>
 
 
-struct MooFileDialogPrivate {
+struct _MooFileDialogPrivate {
     gboolean multiple;
     char *title;
     char *current_dir;
@@ -117,12 +125,12 @@ moo_file_dialog_set_property (GObject        *object,
             break;
 
         case PROP_TYPE:
-            dialog->priv->type = g_value_get_enum (value);
+            dialog->priv->type = (MooFileDialogType) g_value_get_enum (value);
             g_object_notify (object, "type");
             break;
 
         case PROP_PARENT:
-            dialog->priv->parent = g_value_get_object (value);
+            dialog->priv->parent = GTK_WIDGET (g_value_get_object (value));
             g_object_notify (object, "parent");
             break;
 
@@ -247,35 +255,38 @@ moo_file_dialog_class_init (MooFileDialogClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_MULTIPLE,
         g_param_spec_boolean ("multiple", "multiple", "multiple",
-                              FALSE, G_PARAM_READWRITE));
+                              FALSE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_ENABLE_ENCODINGS,
         g_param_spec_boolean ("enable-encodings", "enable-encodings", "enable-encodings",
-                              FALSE, G_PARAM_READWRITE));
+                              FALSE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_TITLE,
         g_param_spec_string ("title", "title", "title",
-                             NULL, G_PARAM_READWRITE));
+                             NULL, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_CURRENT_FOLDER_URI,
         g_param_spec_string ("current-folder-uri", "current-folder-uri",
-                             "current-folder-uri", NULL, G_PARAM_READWRITE));
+                             "current-folder-uri", NULL,
+                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_NAME,
-        g_param_spec_string ("name", "name", "name", NULL, G_PARAM_READWRITE));
+        g_param_spec_string ("name", "name", "name", NULL,
+                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_PARENT,
         g_param_spec_object ("parent", "parent", "parent",
-                             GTK_TYPE_WIDGET, G_PARAM_READWRITE));
+                             GTK_TYPE_WIDGET,
+                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_TYPE,
         g_param_spec_enum ("type", "type", "type",
                            MOO_TYPE_FILE_DIALOG_TYPE, MOO_FILE_DIALOG_OPEN,
-                           G_PARAM_READWRITE));
+                           (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_FILTER_MGR_ID,
         g_param_spec_string ("filter-mgr-id", "filter-mgr-id", "filter-mgr-id",
-                             NULL, G_PARAM_READWRITE));
+                             NULL, (GParamFlags) G_PARAM_READWRITE));
 }
 
 
@@ -326,15 +337,23 @@ moo_file_dialog (GtkWidget  *parent,
 {
     static char *filename;
     MooFileDialog *dialog;
+    GFile *start;
+    GFile *file;
 
-    dialog = moo_file_dialog_new (type, parent, FALSE, title, start_dir, start_name);
+    start = start_dir ? g_file_new_for_path (start_dir) : NULL;
+
+    dialog = moo_file_dialog_new (type, parent, FALSE, title, start, start_name);
     g_return_val_if_fail (dialog != NULL, NULL);
 
     moo_file_dialog_run (dialog);
 
-    g_free (filename);
-    filename = g_strdup (moo_file_dialog_get_filename (dialog));
+    file = moo_file_dialog_get_file (dialog);
 
+    g_free (filename);
+    filename = file ? g_file_get_path (file) : NULL;
+
+    g_object_unref (file);
+    g_object_unref (start);
     g_object_unref (dialog);
     return filename;
 }
@@ -379,7 +398,7 @@ moo_file_dialog_create_widget (MooFileDialog *dialog)
             break;
 
         default:
-            g_critical ("%s: incorrect dialog type specified", G_STRLOC);
+            g_critical ("incorrect dialog type specified");
             return NULL;
     }
 
@@ -407,7 +426,8 @@ moo_file_dialog_create_widget (MooFileDialog *dialog)
                             FALSE, FALSE, 0);
 
     if (dialog->priv->size_prefs_key)
-        _moo_window_set_remember_size (GTK_WINDOW (widget), dialog->priv->size_prefs_key, TRUE);
+        _moo_window_set_remember_size (GTK_WINDOW (widget),
+                                       dialog->priv->size_prefs_key, -1, -1, TRUE);
     if (dialog->priv->parent)
         moo_window_set_parent (widget, dialog->priv->parent);
 
@@ -428,29 +448,29 @@ string_slist_to_strv (GSList *list)
     uris = g_new (char*, len + 1);
 
     for (i = 0; i < len; i++, list = list->next)
-        uris[i] = g_strdup (list->data);
+        uris[i] = g_strdup ((const char*) list->data);
 
     uris[i] = NULL;
     return uris;
 }
 
-static char **
-uri_list_to_filenames (GSList *list)
+static MooFileArray *
+uri_list_to_files (GSList *list)
 {
-    guint len, i;
-    char **filenames;
+    MooFileArray *flocs;
 
     if (!list)
         return NULL;
 
-    len = g_slist_length (list);
-    filenames = g_new (char*, len + 1);
+    flocs = moo_file_array_new ();
 
-    for (i = 0; i < len; i++, list = list->next)
-        filenames[i] = g_filename_from_uri (list->data, NULL, NULL); /* XXX check */
+    while (list)
+    {
+        moo_file_array_take (flocs, g_file_new_for_uri (list->data));
+        list = list->next;
+    }
 
-    filenames[i] = NULL;
-    return filenames;
+    return flocs;
 }
 
 
@@ -616,7 +636,7 @@ moo_file_dialog_run (MooFileDialog *dialog)
 
                 if (!uri_is_valid (uri, &msg))
                 {
-                    moo_error_dialog (filechooser, msg, NULL);
+                    moo_error_dialog (msg, NULL, filechooser);
                     g_free (uri);
                     g_free (msg);
                     continue;
@@ -627,9 +647,9 @@ moo_file_dialog_run (MooFileDialog *dialog)
                     if (g_file_test (filename, G_FILE_TEST_EXISTS) &&
                         !g_file_test (filename, G_FILE_TEST_IS_REGULAR))
                     {
-                        moo_error_dialog (filechooser,
-                                          "Choosen file is not a regular file",
-                                          NULL);
+                        moo_error_dialog (_("Selected file is not a regular file"),
+                                          NULL,
+                                          filechooser);
                         g_free (filename);
                         g_free (uri);
                         continue;
@@ -644,7 +664,7 @@ moo_file_dialog_run (MooFileDialog *dialog)
                         char *display_dirname = g_filename_display_name (dirname);
                         gboolean overwrite;
 
-                        overwrite = moo_overwrite_file_dialog (filechooser, display_name, display_dirname);
+                        overwrite = moo_overwrite_file_dialog (display_name, display_dirname, filechooser);
 
                         g_free (basename);
                         g_free (dirname);
@@ -667,7 +687,7 @@ moo_file_dialog_run (MooFileDialog *dialog)
             }
 
         default:
-            g_critical ("%s: incorrect dialog type specified", G_STRLOC);
+            g_critical ("incorrect dialog type specified");
     }
 
 out:
@@ -711,33 +731,32 @@ moo_file_dialog_set_extra_widget (MooFileDialog *dialog,
     g_return_if_fail (!widget || GTK_IS_WIDGET (widget));
 
     if (widget)
-        MOO_OBJECT_REF_SINK (widget);
+        g_object_ref_sink (widget);
     if (dialog->priv->extra_widget)
         g_object_unref (dialog->priv->extra_widget);
     dialog->priv->extra_widget = widget;
 }
 
 
-const char *
-moo_file_dialog_get_filename (MooFileDialog *dialog)
+GFile *
+moo_file_dialog_get_file (MooFileDialog *dialog)
 {
     g_return_val_if_fail (MOO_IS_FILE_DIALOG (dialog), NULL);
-    return dialog->priv->uri ?
-        g_filename_from_uri (dialog->priv->uri, NULL, NULL) : NULL;
+    return dialog->priv->uri ? g_file_new_for_uri (dialog->priv->uri) : NULL;
 }
 
-const char *
+char *
 moo_file_dialog_get_uri (MooFileDialog *dialog)
 {
     g_return_val_if_fail (MOO_IS_FILE_DIALOG (dialog), NULL);
-    return dialog->priv->uri;
+    return g_strdup (dialog->priv->uri);
 }
 
-char **
-moo_file_dialog_get_filenames (MooFileDialog *dialog)
+MooFileArray *
+moo_file_dialog_get_files (MooFileDialog *dialog)
 {
     g_return_val_if_fail (MOO_IS_FILE_DIALOG (dialog), NULL);
-    return uri_list_to_filenames (dialog->priv->uris);
+    return uri_list_to_files (dialog->priv->uris);
 }
 
 char **
@@ -800,23 +819,24 @@ moo_file_dialog_new (MooFileDialogType type,
                      GtkWidget      *parent,
                      gboolean        multiple,
                      const char     *title,
-                     const char     *start_dir,
+                     GFile          *start_loc,
                      const char     *start_name)
 {
     char *start_dir_uri = NULL;
     MooFileDialog *dialog;
 
-    if (start_dir)
-        start_dir_uri = g_filename_to_uri (start_dir, NULL, NULL);
+    if (start_loc)
+        start_dir_uri = g_file_get_uri (start_loc);
 
-    dialog = g_object_new (MOO_TYPE_FILE_DIALOG,
+    dialog = MOO_FILE_DIALOG (
+             g_object_new (MOO_TYPE_FILE_DIALOG,
                            "parent", parent,
                            "type", type,
                            "multiple", multiple,
                            "title", title,
                            "current-folder-uri", start_dir_uri,
                            "name", start_name,
-                           NULL);
+                           (const char*) NULL));
 
     g_free (start_dir_uri);
     return dialog;
