@@ -1,7 +1,7 @@
 /*
  *   moofilewatch.h
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -44,9 +44,11 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-mem.h"
 #include "mooutils/moofilewatch.h"
+#include "mooutils/moostat.h"
 #include "mooutils/mootype-macros.h"
 #include "marshals.h"
 #include "mooutils/mooutils-thread.h"
+#include "mooutils/moolist.h"
 
 
 #if 1
@@ -90,6 +92,8 @@ struct WatchFuncs {
                                  Monitor        *monitor);
 };
 
+MOO_DEFINE_SLIST(MonitorList, monitor_list, Monitor)
+
 struct _MooFileWatch {
     guint ref_count;
     guint id;
@@ -98,8 +102,8 @@ struct _MooFileWatch {
     FAMConnection fam_connection;
     guint fam_connection_watch;
 #endif
-    GSList      *monitors;
-    GHashTable  *requests;  /* int -> Monitor* */
+    MonitorList *monitors;
+    GHashTable *requests;  /* int -> Monitor* */
     guint alive : 1;
 };
 
@@ -227,12 +231,12 @@ moo_file_watch_unref (MooFileWatch *watch)
 
     if (watch->alive)
     {
-        g_warning ("%s: finalizing open watch", G_STRLOC);
+        g_warning ("finalizing open watch");
 
         if (!moo_file_watch_close (watch, &error))
         {
-            g_warning ("%s: error in moo_file_watch_close()", G_STRLOC);
-            g_warning ("%s: %s", G_STRLOC, error->message);
+            g_warning ("error in moo_file_watch_close(): %s",
+                       moo_error_message (error));
             g_error_free (error);
         }
     }
@@ -315,7 +319,7 @@ gboolean
 moo_file_watch_close (MooFileWatch   *watch,
                       GError        **error)
 {
-    GSList *monitors;
+    MonitorList *monitors;
 
     g_return_val_if_fail (watch != NULL, FALSE);
 
@@ -334,7 +338,7 @@ moo_file_watch_close (MooFileWatch   *watch,
             watch_funcs.stop_monitor (watch, mon);
 
         monitor_free (mon);
-        monitors = g_slist_delete_link (monitors, monitors);
+        monitors = monitor_list_delete_link (monitors, monitors);
     }
 
     return watch_funcs.shutdown (watch, error);
@@ -373,7 +377,7 @@ moo_file_watch_create_monitor (MooFileWatch   *watch,
     }
 
     monitor->alive = TRUE;
-    watch->monitors = g_slist_prepend (watch->monitors, monitor);
+    watch->monitors = monitor_list_prepend (watch->monitors, monitor);
     g_hash_table_insert (watch->requests, GUINT_TO_POINTER (monitor->id), monitor);
 
     DEBUG_PRINT ("created monitor %d for '%s'", monitor->id, monitor->filename);
@@ -390,11 +394,11 @@ moo_file_watch_cancel_monitor (MooFileWatch *watch,
 
     g_return_if_fail (watch != NULL);
 
-    monitor = g_hash_table_lookup (watch->requests,
-                                   GUINT_TO_POINTER (monitor_id));
+    monitor = (Monitor*) g_hash_table_lookup (watch->requests,
+                                              GUINT_TO_POINTER (monitor_id));
     g_return_if_fail (monitor != NULL);
 
-    watch->monitors = g_slist_remove (watch->monitors, monitor);
+    watch->monitors = monitor_list_remove (watch->monitors, monitor);
     g_hash_table_remove (watch->requests, GUINT_TO_POINTER (monitor->id));
 
     if (monitor->alive)
@@ -668,7 +672,7 @@ fam_event_to_file_event (FAMEvent     *fe,
             return NULL;
 
         default:
-            g_warning ("%s: unknown FAM code %d", G_STRLOC, fe->code);
+            g_warning ("unknown FAM code %d", fe->code);
             return NULL;
     }
 
@@ -695,8 +699,8 @@ read_fam_events (G_GNUC_UNUSED GIOChannel *source,
 
         if (watch->alive && !moo_file_watch_close (watch, &error))
         {
-            g_warning ("%s: error in moo_file_watch_close()", G_STRLOC);
-            g_warning ("%s: %s", G_STRLOC, error->message);
+            g_warning ("error in moo_file_watch_close(): %s",
+                       moo_error_message (error));
             g_error_free (error);
         }
 
@@ -742,8 +746,7 @@ read_fam_events (G_GNUC_UNUSED GIOChannel *source,
 
         if (!moo_file_watch_close (watch, &error))
         {
-            g_warning ("%s: error in moo_file_watch_close()", G_STRLOC);
-            g_warning ("%s: %s", G_STRLOC, error->message);
+            g_warning ("%s", moo_error_message (error));
             g_error_free (error);
         }
 
@@ -777,10 +780,10 @@ watch_stat_start (MooFileWatch *watch,
                   G_GNUC_UNUSED GError **error)
 {
     watch->stat_timeout =
-            _moo_timeout_add_full (MOO_STAT_PRIORITY,
-                                   MOO_STAT_TIMEOUT,
-                                   (GSourceFunc) do_stat,
-                                   watch, NULL);
+            gdk_threads_add_timeout_full (MOO_STAT_PRIORITY,
+                                          MOO_STAT_TIMEOUT,
+                                          (GSourceFunc) do_stat,
+                                          watch, NULL);
     return TRUE;
 }
 
@@ -808,7 +811,7 @@ watch_stat_start_monitor (MooFileWatch   *watch,
 
     errno = 0;
 
-    if (g_stat (monitor->filename, &buf) != 0)
+    if (moo_stat (monitor->filename, &buf) != 0)
     {
         int saved_errno = errno;
         g_set_error (error, MOO_FILE_WATCH_ERROR,
@@ -841,8 +844,8 @@ watch_stat_start_monitor (MooFileWatch   *watch,
 static gboolean
 do_stat (MooFileWatch *watch)
 {
-    GSList *l;
-    GSList *list = NULL;
+    MonitorList *lm;
+    GSList *list = NULL, *lid;
     GSList *to_remove = NULL;
     gboolean result = TRUE;
 
@@ -853,21 +856,21 @@ do_stat (MooFileWatch *watch)
     if (!watch->monitors)
         goto out;
 
-    for (l = watch->monitors; l != NULL; l = l->next)
+    for (lm = watch->monitors; lm != NULL; lm = lm->next)
     {
-        Monitor *m = l->data;
+        Monitor *m = lm->data;
         list = g_slist_prepend (list, GUINT_TO_POINTER (m->id));
     }
 
     /* Order of list is correct now, watch->monitors is last-added-first */
-    for (l = list; l != NULL; l = l->next)
+    for (lid = list; lid != NULL; lid = lid->next)
     {
         gboolean do_emit = FALSE;
         MooFileEvent event;
         Monitor *monitor;
         time_t old;
 
-        monitor = g_hash_table_lookup (watch->requests, l->data);
+        monitor = (Monitor*) g_hash_table_lookup (watch->requests, lid->data);
 
         if (!monitor || !monitor->alive)
             continue;
@@ -880,7 +883,7 @@ do_stat (MooFileWatch *watch)
         event.filename = monitor->filename;
         event.error = NULL;
 
-        if (g_stat (monitor->filename, &monitor->statbuf) != 0)
+        if (moo_stat (monitor->filename, &monitor->statbuf) != 0)
         {
             if (errno == ENOENT)
             {
@@ -913,9 +916,9 @@ do_stat (MooFileWatch *watch)
             g_error_free (event.error);
     }
 
-    for (l = to_remove; l != NULL; l = l->next)
-        if (g_hash_table_lookup (watch->requests, GUINT_TO_POINTER (l->data)))
-            moo_file_watch_cancel_monitor (watch, GPOINTER_TO_UINT (l->data));
+    for (lid = to_remove; lid != NULL; lid = lid->next)
+        if (g_hash_table_lookup (watch->requests, GUINT_TO_POINTER (lid->data)))
+            moo_file_watch_cancel_monitor (watch, GPOINTER_TO_UINT (lid->data));
 
     g_slist_free (to_remove);
     g_slist_free (list);
@@ -1043,7 +1046,7 @@ fam_thread_add_path (FAMThread  *thr,
 
     if (thr->n_events >= MAXIMUM_WAIT_OBJECTS)
     {
-        g_critical ("%s: too many folders watched", G_STRLOC);
+        g_critical ("too many folders watched");
         return;
     }
 
@@ -1059,7 +1062,7 @@ fam_thread_add_path (FAMThread  *thr,
     }
 
     thr->events[thr->n_events] =
-        FindFirstChangeNotificationW (win_path, FALSE,
+        FindFirstChangeNotificationW ((const WCHAR*) win_path, FALSE,
                                       FILE_NOTIFY_CHANGE_FILE_NAME |
                                       FILE_NOTIFY_CHANGE_DIR_NAME);
 
@@ -1111,8 +1114,8 @@ fam_thread_remove_path (FAMThread  *thr,
         }
     }
 
-    g_critical ("%s: can't remove watch %d, request %d",
-                G_STRLOC, watch_id, request);
+    g_critical ("can't remove watch %d, request %d",
+                watch_id, request);
 }
 
 static void
@@ -1123,7 +1126,7 @@ fam_thread_check_dir (FAMThread *thr,
 
     errno = 0;
 
-    if (g_stat (thr->watches[idx].path, &buf) != 0 &&
+    if (moo_stat (thr->watches[idx].path, &buf) != 0 &&
         errno == ENOENT)
     {
         fam_thread_event (MOO_FILE_EVENT_DELETED,
@@ -1168,7 +1171,7 @@ fam_thread_do_command (FAMThread *thr)
     DEBUG_PRINT ("fam_thread_do_command start");
 
     g_mutex_lock (thr->lock);
-    while ((cmd = g_async_queue_try_pop (thr->incoming)))
+    while ((cmd = (FAMThreadCommand*) g_async_queue_try_pop (thr->incoming)))
         list = g_slist_prepend (list, cmd);
     ResetEvent (thr->events[0]);
     g_mutex_unlock (thr->lock);
@@ -1177,7 +1180,7 @@ fam_thread_do_command (FAMThread *thr)
 
     while (list)
     {
-        cmd = list->data;
+        cmd = (FAMThreadCommand*) list->data;
 
         switch (cmd->type)
         {
@@ -1213,7 +1216,7 @@ fam_thread_main (FAMThread *thr)
         if (ret == (int) WAIT_FAILED)
         {
             char *msg = g_win32_error_message (GetLastError ());
-            g_critical ("%s: %s", G_STRLOC, msg);
+            g_critical ("%s", msg);
             g_free (msg);
             break;
         }
@@ -1224,7 +1227,7 @@ fam_thread_main (FAMThread *thr)
             fam_thread_check_dir (thr, ret - WAIT_OBJECT_0);
         else
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
             break;
         }
     }
@@ -1264,7 +1267,7 @@ find_watch (guint id)
 
     for (l = fam->watches; l != NULL; l = l->next)
     {
-        MooFileWatch *watch = l->data;
+        MooFileWatch *watch = (MooFileWatch*) l->data;
 
         if (watch->id == id)
             return watch;
@@ -1288,8 +1291,8 @@ do_event (FAMEvent *event)
         return;
     }
 
-    monitor = g_hash_table_lookup (watch->requests,
-                                   GUINT_TO_POINTER (event->request));
+    monitor = (Monitor*)
+        g_hash_table_lookup (watch->requests, GUINT_TO_POINTER (event->request));
 
     if (!monitor)
     {
@@ -1312,7 +1315,7 @@ do_event (FAMEvent *event)
     else
     {
         DEBUG_PRINT ("got event for filename %s", monitor->filename);
-        watch_event.code = event->code;
+        watch_event.code = (MooFileEventCode) event->code;
     }
 
     moo_file_watch_emit_event (watch, &watch_event, monitor);
@@ -1329,15 +1332,15 @@ event_callback (GList *events)
     while (events)
     {
         GList *l;
-        FAMEvent *event = events->data;
+        FAMEvent *event = (FAMEvent*) events->data;
         gboolean found = FALSE;
 
-        event = events->data;
+        event = (FAMEvent*) events->data;
         events = events->next;
 
         for (l = trimmed; l != NULL; l = l->next)
         {
-            FAMEvent *old_event = l->data;
+            FAMEvent *old_event = (FAMEvent*) l->data;
 
             if (old_event->watch_id == event->watch_id &&
                 old_event->request == event->request)
@@ -1359,7 +1362,7 @@ event_callback (GList *events)
 
     while (trimmed)
     {
-        do_event (trimmed->data);
+        do_event ((FAMEvent*) trimmed->data);
         trimmed = g_list_delete_link (trimmed, trimmed);
     }
 }
@@ -1380,8 +1383,7 @@ fam_win32_init (void)
         if (!fam->thread_data.events[0])
         {
             char *msg = g_win32_error_message (GetLastError ());
-            g_critical ("%s: could not create incoming event", G_STRLOC);
-            g_critical ("%s: %s", G_STRLOC, msg);
+            g_critical ("could not create incoming event: %s", msg);
             g_free (msg);
             return FALSE;
         }
@@ -1393,8 +1395,7 @@ fam_win32_init (void)
 
         if (!g_thread_create ((GThreadFunc) fam_thread_main, &fam->thread_data, FALSE, &error))
         {
-            g_critical ("%s: could not start watch thread", G_STRLOC);
-            g_critical ("%s: %s", G_STRLOC, error->message);
+            g_critical ("could not start watch thread: %s", moo_error_message (error));
             g_error_free (error);
         }
         else
@@ -1450,7 +1451,7 @@ watch_win32_start_monitor (MooFileWatch   *watch,
 
     errno = 0;
 
-    if (g_stat (monitor->filename, &buf) != 0)
+    if (moo_stat (monitor->filename, &buf) != 0)
     {
         int saved_errno = errno;
         g_set_error (error, MOO_FILE_WATCH_ERROR,
