@@ -12,68 +12,68 @@ struct ListType {                                                       \
     ListType *next;                                                     \
 };                                                                      \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_from_g##glisttype (GListType *list)                         \
 {                                                                       \
     return (ListType*) list;                                            \
 }                                                                       \
                                                                         \
-inline static GListType *                                               \
+G_INLINE_FUNC GListType *                                               \
 list_type##_to_g##glisttype (ListType *list)                            \
 {                                                                       \
     return (GListType*) list;                                           \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 list_type##_free_links (ListType *list)                                 \
 {                                                                       \
     g_##glisttype##_free ((GListType*) list);                           \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_copy_links (ListType *list)                                 \
 {                                                                       \
     return (ListType*) g_##glisttype##_copy ((GListType*) list);        \
 }                                                                       \
                                                                         \
-inline static guint                                                     \
+G_INLINE_FUNC guint                                                     \
 list_type##_length (ListType *list)                                     \
 {                                                                       \
     return g_##glisttype##_length ((GListType*) list);                  \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_reverse (ListType *list)                                    \
 {                                                                       \
     return (ListType*) g_##glisttype##_reverse ((GListType*) list);     \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_prepend (ListType *list, ElmType *data)                     \
 {                                                                       \
     return (ListType*) g_##glisttype##_prepend ((GListType*)list, data);\
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_append (ListType *list, ElmType *data)                      \
 {                                                                       \
     return (ListType*) g_##glisttype##_append ((GListType*)list, data); \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_concat (ListType *list1, ListType *list2)                   \
 {                                                                       \
     return (ListType*) g_##glisttype##_concat ((GListType*) list1,      \
                                        (GListType*) list2);             \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_remove (ListType *list, const ElmType *data)                \
 {                                                                       \
     return (ListType*) g_##glisttype##_remove ((GListType*)list, data); \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_delete_link (ListType *list, ListType *link)                \
 {                                                                       \
     return (ListType*)                                                  \
@@ -81,7 +81,7 @@ list_type##_delete_link (ListType *list, ListType *link)                \
                                      (GListType*)link);                 \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_find (ListType *list, const ElmType *data)                  \
 {                                                                       \
     while (list)                                                        \
@@ -93,7 +93,7 @@ list_type##_find (ListType *list, const ElmType *data)                  \
     return NULL;                                                        \
 }                                                                       \
                                                                         \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_find_custom (ListType      *list,                           \
                          gconstpointer  data,                           \
                          GCompareFunc   func)                           \
@@ -104,7 +104,7 @@ list_type##_find_custom (ListType      *list,                           \
                                                                         \
 typedef void (*ListType##Func) (ElmType *data, gpointer user_data);     \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 list_type##_foreach (ListType      *list,                               \
                      ListType##Func func,                               \
                      gpointer       user_data)                          \
@@ -114,7 +114,7 @@ list_type##_foreach (ListType      *list,                               \
 }
 
 #define MOO_DEFINE_LIST_COPY_FUNC(ListType, list_type, elm_copy_func)   \
-inline static ListType *                                                \
+G_INLINE_FUNC ListType *                                                \
 list_type##_copy (ListType *list)                                       \
 {                                                                       \
     ListType *copy = NULL;                                              \
@@ -127,7 +127,7 @@ list_type##_copy (ListType *list)                                       \
 }
 
 #define MOO_DEFINE_LIST_FREE_FUNC(ListType, list_type, elm_free_func)   \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 list_type##_free (ListType *list)                                       \
 {                                                                       \
     ListType *l;                                                        \
@@ -156,7 +156,7 @@ list_type##_free (ListType *list)                                       \
 
 #ifdef __cplusplus
 #define _MOO_DEFINE_QUEUE_FOREACH(Element, element)                     \
-template<typename T> inline static void                                 \
+template<typename T> G_INLINE_FUNC void                                 \
 element##_queue_foreach (Element##Queue *queue,                         \
                          void (*func) (Element *elm, T *data),          \
                          T *data)                                       \
@@ -165,7 +165,7 @@ element##_queue_foreach (Element##Queue *queue,                         \
                      (GFunc) func, data);                               \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_foreach (Element##Queue *queue,                         \
                          void (*func) (Element *elm),                   \
                          void *)                                        \
@@ -175,7 +175,7 @@ element##_queue_foreach (Element##Queue *queue,                         \
 }
 #else
 #define _MOO_DEFINE_QUEUE_FOREACH(Element, element)                     \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_foreach (Element##Queue *queue,                         \
                          void (*func) (Element *elm, void *data),       \
                          void *data)                                    \
@@ -196,43 +196,43 @@ struct Element##Queue {                                                 \
   guint length;                                                         \
 };                                                                      \
                                                                         \
-inline static Element##Queue *                                          \
+G_INLINE_FUNC Element##Queue *                                          \
 element##_queue_from_gqueue (GQueue *queue)                             \
 {                                                                       \
     return (Element##Queue*) queue;                                     \
 }                                                                       \
                                                                         \
-inline static GQueue *                                                  \
+G_INLINE_FUNC GQueue *                                                  \
 element##_queue_to_gqueue (Element##Queue *queue)                       \
 {                                                                       \
     return (GQueue*) queue;                                             \
 }                                                                       \
                                                                         \
-inline static Element##Queue *                                          \
+G_INLINE_FUNC Element##Queue *                                          \
 element##_queue_new (void)                                              \
 {                                                                       \
     return element##_queue_from_gqueue (g_queue_new ());                \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_free_links (Element##Queue *queue)                      \
 {                                                                       \
     g_queue_free (element##_queue_to_gqueue (queue));                   \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_push_head (Element##Queue *queue, Element *elm)         \
 {                                                                       \
     g_queue_push_head (element##_queue_to_gqueue (queue), elm);         \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_push_tail (Element##Queue *queue, Element *elm)         \
 {                                                                       \
     g_queue_push_tail (element##_queue_to_gqueue (queue), elm);         \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_push_head_link (Element##Queue *queue,                  \
                                 Element##List  *link_)                  \
 {                                                                       \
@@ -240,7 +240,7 @@ element##_queue_push_head_link (Element##Queue *queue,                  \
                             element##_list_to_glist (link_));           \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_push_tail_link (Element##Queue *queue,                  \
                                 Element##List  *link_)                  \
 {                                                                       \
@@ -248,7 +248,7 @@ element##_queue_push_tail_link (Element##Queue *queue,                  \
                             element##_list_to_glist (link_));           \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_unlink (Element##Queue *queue,                          \
                         Element##List  *link_)                          \
 {                                                                       \
@@ -256,7 +256,7 @@ element##_queue_unlink (Element##Queue *queue,                          \
                     element##_list_to_glist (link_));                   \
 }                                                                       \
                                                                         \
-inline static void                                                      \
+G_INLINE_FUNC void                                                      \
 element##_queue_delete_link (Element##Queue *queue,                     \
                              Element##List  *link_)                     \
 {                                                                       \
