@@ -1,22 +1,30 @@
 /*
  *   mooprefsdialog.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooPrefsDialog: (parent GtkDialog) (constructable) (moo.private 1)
+ **/
+
 #include "config.h"
 #include "marshals.h"
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/moohelp.h"
 #include "mooutils/mooutils-treeview.h"
-#include "help-sections.h"
+#include "mooutils/moocompat.h"
+#include "moo-help-sections.h"
 
 
 enum {
@@ -47,7 +55,7 @@ static void setup_pages_list                (MooPrefsDialog *dialog);
 static void pages_list_selection_changed    (MooPrefsDialog *dialog,
                                              GtkTreeSelection *selection);
 
-static void init_page                       (MooPrefsDialogPage *page);
+static void init_page                       (MooPrefsPage   *page);
 
 static gboolean moo_prefs_dialog_help       (GtkWidget      *widget);
 
@@ -92,7 +100,7 @@ moo_prefs_dialog_class_init (MooPrefsDialogClass *klass)
                                             "hide_on_delete",
                                             "Hide on delete",
                                             FALSE,
-                                            G_PARAM_READWRITE));
+                                            (GParamFlags) G_PARAM_READWRITE));
 
     prefs_dialog_signals[APPLY] =
         g_signal_new ("apply",
@@ -122,7 +130,7 @@ moo_prefs_dialog_init (MooPrefsDialog *dialog)
     moo_help_set_func (GTK_WIDGET (dialog), moo_prefs_dialog_help);
     moo_help_connect_keys (GTK_WIDGET (dialog));
 
-    _moo_window_set_remember_size (GTK_WINDOW (dialog), "Dialogs/Preferences", FALSE);
+    _moo_window_set_remember_size (GTK_WINDOW (dialog), "Dialogs/Preferences", -1, -1, FALSE);
 
     gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 
@@ -159,7 +167,7 @@ moo_prefs_dialog_init (MooPrefsDialog *dialog)
     notebook = gtk_notebook_new ();
     gtk_widget_show (notebook);
     gtk_box_pack_start (GTK_BOX (hbox), notebook, TRUE, TRUE, 0);
-    GTK_WIDGET_UNSET_FLAGS (notebook, GTK_CAN_FOCUS);
+    GTK_WIDGET_UNSET_CAN_FOCUS (notebook);
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
     gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 
@@ -218,7 +226,7 @@ setup_pages_list (MooPrefsDialog *dialog)
                                         GDK_TYPE_PIXBUF,
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
-                                        MOO_TYPE_PREFS_DIALOG_PAGE);
+                                        MOO_TYPE_PREFS_PAGE);
     tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (dialog->store));
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
     gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree), FALSE);
@@ -259,7 +267,7 @@ pages_list_selection_changed (MooPrefsDialog *dialog,
 
     if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-        MooPrefsDialogPage *page = NULL;
+        MooPrefsPage *page = NULL;
         gtk_tree_model_get (model, &iter, PAGE_COLUMN, &page, -1);
         g_return_if_fail (page != NULL);
 
@@ -271,24 +279,24 @@ pages_list_selection_changed (MooPrefsDialog *dialog,
     }
     else
     {
-        g_critical ("%s: nothing selected", G_STRLOC);
+        g_critical ("nothing selected");
 
         if (gtk_tree_model_get_iter_first (model, &iter))
             gtk_tree_selection_select_iter (selection, &iter);
         else
-            g_critical ("%s: list is empty", G_STRLOC);
+            g_critical ("list is empty");
     }
 }
 
 
 static gboolean
-page_initialized (MooPrefsDialogPage *page)
+page_initialized (MooPrefsPage *page)
 {
     return g_object_get_data (G_OBJECT (page), "moo-prefs-dialog-page-initialized") != NULL;
 }
 
 static void
-init_page (MooPrefsDialogPage *page)
+init_page (MooPrefsPage *page)
 {
     if (!g_object_get_data (G_OBJECT (page), "moo-prefs-dialog-page-initialized"))
     {
@@ -350,7 +358,8 @@ GtkWidget*
 moo_prefs_dialog_new (const char *title)
 {
     return GTK_WIDGET (g_object_new (MOO_TYPE_PREFS_DIALOG,
-                                     "title", title, NULL));
+                                     "title", title,
+                                     (const char*) NULL));
 }
 
 
@@ -432,12 +441,10 @@ moo_prefs_dialog_apply (MooPrefsDialog *dialog)
 
     while (list)
     {
-        MooPrefsDialogPage *page = list->data;
+        MooPrefsPage *page = list->data;
 
-        if (!(GTK_OBJECT_FLAGS (page) & GTK_IN_DESTRUCTION) &&
-            page->auto_apply &&
-            page_initialized (page))
-                g_signal_emit_by_name (page, "apply");
+        if (page->auto_apply && page_initialized (page))
+            g_signal_emit_by_name (page, "apply");
 
         g_object_unref (page);
         list = g_slist_delete_link (list, list);
@@ -465,7 +472,7 @@ moo_prefs_dialog_insert_page (MooPrefsDialog     *dialog,
     GtkTreePath *path = NULL;
 
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
-    g_return_if_fail (MOO_IS_PREFS_DIALOG_PAGE (page));
+    g_return_if_fail (MOO_IS_PREFS_PAGE (page));
     g_return_if_fail (page->parent == NULL);
 
     if (position < 0)
@@ -519,7 +526,7 @@ moo_prefs_dialog_remove_page (MooPrefsDialog     *dialog,
     GtkTreeIter iter;
 
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
-    g_return_if_fail (MOO_IS_PREFS_DIALOG_PAGE (page));
+    g_return_if_fail (MOO_IS_PREFS_PAGE (page));
 
     ref = g_object_get_data (G_OBJECT (page), "moo-prefs-dialog-row");
     g_return_if_fail (ref && gtk_tree_row_reference_valid (ref));
