@@ -13,7 +13,6 @@
  *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define MOOEDIT_COMPILATION
 #include "moooutputfilterregex.h"
 #include "moocommand.h"
 #include "moocmdview.h"
@@ -23,7 +22,7 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moomarkup.h"
 #include "mooutils/mooi18n.h"
-#include <glib/gregex.h>
+#include <glib.h>
 #include <string.h>
 
 #define FILTERS_VERSION     "1.0"
