@@ -16,7 +16,7 @@
 #ifndef MOO_USER_TOOLS_PREFS_H
 #define MOO_USER_TOOLS_PREFS_H
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
